@@ -1,7 +1,7 @@
 /*
  *      mmguicore.c
  *      
- *      Copyright 2013-2014 Alex <alex@linuxonly.ru>
+ *      Copyright 2013-2015 Alex <alex@linuxonly.ru>
  *      
  *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -34,6 +34,8 @@
 #include "smsdb.h"
 #include "trafficdb.h"
 #include "netlink.h"
+#include "polkit.h"
+#include "svcmanager.h"
 #include "resources.h"
 
 /*Module*/
@@ -43,9 +45,11 @@
 #define MMGUICORE_CACHE_DIR              "modem-manager-gui"
 #define MMGUICORE_CACHE_FILE             "modules.conf"
 #define MMGUICORE_CACHE_PERM             0755
+#define MMGUICORE_CACHE_VER              3
 /*Cache file sections*/
 #define MMGUICORE_FILE_ROOT_SECTION      "cache"
 #define MMGUICORE_FILE_TIMESTAMP         "timestamp"
+#define MMGUICORE_FILE_VERSION           "version"
 /*Module entry*/
 #define MMGUICORE_FILE_IDENTIFIER        "identifier"
 #define MMGUICORE_FILE_TYPE              "type"
@@ -54,6 +58,7 @@
 #define MMGUICORE_FILE_FUNCTIONS         "functions"
 #define MMGUICORE_FILE_DESCRIPTION       "description"
 #define MMGUICORE_FILE_SERVICENAME       "servicename"
+#define MMGUICORE_FILE_SYSTEMDNAME       "systemdname"
 /*SMS parameters*/
 #define MMGUI_MIN_SMS_NUMBER_LENGTH      3
 #define MMGUI_MAX_SMS_NUMBER_LENGTH      20
@@ -63,10 +68,12 @@
 /*Commands*/
 #define MMGUI_THREAD_STOP_CMD            0x00
 
-enum _mmgui_core_dbus_start_reply_status {
-	MMGUI_CORE_DBUS_START_REPLY_ALREADY_RUNNING 	= 2,
-	MMGUI_CORE_DBUS_START_REPLY_SUCCESS 	        = 1
-};
+
+
+static gboolean mmguicore_modules_enumerate(mmguicore_t mmguicore);
+static gboolean mmguicore_modules_prepare(mmguicore_t mmguicore);
+
+static gboolean mmguicore_main(mmguicore_t mmguicore);
 
 gboolean mmguicore_devices_add(mmguicore_t mmguicore, mmguidevice_t device);
 gboolean mmguicore_devices_remove(mmguicore_t mmguicore, guint deviceid);
@@ -109,6 +116,11 @@ static void mmguicore_event_callback(enum _mmgui_event event, gpointer mmguicore
 				(mmguicorelc->extcb)(event, mmguicorelc, data, mmguicorelc->userdata);
 			}
 			break;
+		case MMGUI_EVENT_DEVICE_PREPARED_STATUS:
+			if (mmguicorelc->extcb != NULL) {
+				(mmguicorelc->extcb)(event, mmguicorelc, data, mmguicorelc->userdata);
+			}
+			break;	
 		case MMGUI_EVENT_DEVICE_CONNECTION_STATUS:
 			if (mmguicorelc->extcb != NULL) {
 				(mmguicorelc->extcb)(event, mmguicorelc, data, mmguicorelc->userdata);
@@ -188,164 +200,94 @@ static void mmguicore_event_callback(enum _mmgui_event event, gpointer mmguicore
 	}
 }
 
-static gboolean mmguicore_system_services_activate(gchar *interface, guint *status)
+static void mmguicore_svcmanager_callback(gpointer svcmanager, gint event, gpointer subject, gpointer userdata)
 {
-	GDBusConnection *connection;
-	GDBusProxy *proxy;
-	gboolean res;
-	GVariant *statusv;
-	GError *error;
-		
-	if (interface == NULL) return FALSE;
+	mmguicore_t mmguicore;
+	GSList *iterator;
+	mmguimodule_t modparams;
+	gchar *modname;
+			
+	mmguicore = (mmguicore_t)userdata;
 	
-	error = NULL;
+	if (mmguicore == NULL) return;
 	
-	connection = g_bus_get_sync(G_BUS_TYPE_SYSTEM, NULL, &error);
-	
-	if ((connection == NULL) && (error != NULL)) {
-		g_printf("Core error: %s\n", error->message);
-		g_error_free(error);
-		return FALSE;
+	switch (event) {
+		case MMGUI_SVCMANGER_EVENT_STARTED:
+			/*Forward event*/
+			if (mmguicore->extcb != NULL) {
+				(mmguicore->extcb)(MMGUI_EVENT_SERVICE_ACTIVATION_STARTED, mmguicore, NULL, mmguicore->userdata);
+			}
+			break;
+		case MMGUI_SVCMANGER_EVENT_ENTITY_ACTIVATED:
+			modname = mmgui_svcmanager_get_transition_module_name(subject);
+			if ((modname != NULL) && (mmguicore->modules != NULL)) {
+				/*Mark module available*/
+				for (iterator=mmguicore->modules; iterator; iterator=iterator->next) {
+					modparams = iterator->data;
+					if (g_str_equal(modparams->shortname, modname)) {
+						modparams->applicable = TRUE;
+						/*Forward event*/
+						if (mmguicore->extcb != NULL) {
+							(mmguicore->extcb)(MMGUI_EVENT_SERVICE_ACTIVATION_SERVICE_ACTIVATED, mmguicore, modparams->description, mmguicore->userdata);
+						}
+						break;
+					}
+				}
+			}
+			break;
+		case MMGUI_SVCMANGER_EVENT_ENTITY_CHANGED:
+			modname = mmgui_svcmanager_get_transition_module_name(subject);
+			if ((modname != NULL) && (mmguicore->modules != NULL)) {
+				/*Mark module available*/
+				for (iterator=mmguicore->modules; iterator; iterator=iterator->next) {
+					modparams = iterator->data;
+					if (g_str_equal(modparams->shortname, modname)) {
+						/*Forward event*/
+						if (mmguicore->extcb != NULL) {
+							(mmguicore->extcb)(MMGUI_EVENT_SERVICE_ACTIVATION_SERVICE_CHANGED, mmguicore, modparams->description, mmguicore->userdata);
+						}
+						break;
+					}
+				}
+			}
+			break;
+		case MMGUI_SVCMANGER_EVENT_ENTITY_ERROR:
+			/*Forward service error event*/
+			if (mmguicore->extcb != NULL) {
+				(mmguicore->extcb)(MMGUI_EVENT_SERVICE_ACTIVATION_SERVICE_ERROR, mmguicore, mmgui_svcmanager_get_last_error(svcmanager), mmguicore->userdata);
+			}
+			break;
+		case MMGUI_SVCMANGER_EVENT_FINISHED:
+			if (mmguicore_main(mmguicore)) {
+				/*Forward event*/
+				if (mmguicore->extcb != NULL) {
+					(mmguicore->extcb)(MMGUI_EVENT_SERVICE_ACTIVATION_FINISHED, mmguicore, NULL, mmguicore->userdata);
+				}
+			} else {
+				/*Forward startup error event*/
+				if (mmguicore->extcb != NULL) {
+					(mmguicore->extcb)(MMGUI_EVENT_SERVICE_ACTIVATION_STARTUP_ERROR, mmguicore, NULL, mmguicore->userdata);
+				}
+			}
+			break;
+		case MMGUI_SVCMANGER_EVENT_AUTH_ERROR:
+			/*Forward authentication error event*/
+			if (mmguicore->extcb != NULL) {
+				(mmguicore->extcb)(MMGUI_EVENT_SERVICE_ACTIVATION_AUTH_ERROR, mmguicore, NULL, mmguicore->userdata);
+			}
+			break;
+		case MMGUI_SVCMANGER_EVENT_OTHER_ERROR:
+			/*Forward unknown error event*/
+			if (mmguicore->extcb != NULL) {
+				(mmguicore->extcb)(MMGUI_EVENT_SERVICE_ACTIVATION_OTHER_ERROR, mmguicore, mmgui_svcmanager_get_last_error(svcmanager), mmguicore->userdata);
+			}
+			break;
+		default:
+			
+			break;
 	}
-	
-	error = NULL;
-	
-	proxy = g_dbus_proxy_new_sync(connection,
-									G_DBUS_PROXY_FLAGS_NONE,
-									NULL,
-									"org.freedesktop.DBus",
-									"/org/freedesktop/DBus",
-									"org.freedesktop.DBus",
-									NULL,
-									&error);
-	
-	if ((proxy == NULL) && (error != NULL)) {
-		g_printf("Core error: %s\n", error->message);
-		g_error_free(error);
-		g_object_unref(connection);
-		return FALSE;
-	}
-	
-	error = NULL;
-	
-	statusv = g_dbus_proxy_call_sync(proxy,
-									"StartServiceByName",
-									g_variant_new("(su)", interface, 0),
-									0,
-									-1,
-									NULL,
-									&error);
-	
-	if ((statusv == NULL) && (error != NULL)) {
-		g_printf("Core error: %s\n", error->message);
-		g_error_free(error);
-		res = FALSE;
-	} else {
-		if (status != NULL) {
-			g_variant_get(statusv, "(u)", status);
-			g_debug("Service start status: %u\n", *status);
-		}
-		res = TRUE;
-	}
-	
-	g_object_unref(proxy);
-	g_object_unref(connection);
-	
-	return res;
 }
 
-static void mmguicore_system_services_destroy_notify(gpointer data)
-{
-	if (data != NULL) g_free(data);
-}
-
-static GHashTable *mmguicore_system_services_enum(gboolean activatable)
-{
-	GDBusConnection *connection;
-	GDBusProxy *proxy;
-	GVariant *services;
-	gchar *command;
-	GError *error;
-	GVariantIter siterl1, siterl2;
-	GVariant *snodel1, *snodel2;
-	gsize snamesize;
-	gchar *servicename;
-	GHashTable *serviceshash;
-	
-	error = NULL;
-	
-	connection = g_bus_get_sync(G_BUS_TYPE_SYSTEM, NULL, &error);
-	
-	if ((connection == NULL) && (error != NULL)) {
-		g_debug("Core error: %s\n", error->message);
-		g_error_free(error);
-		return NULL;
-	}
-	
-	error = NULL;
-	
-	proxy = g_dbus_proxy_new_sync(connection,
-									G_DBUS_PROXY_FLAGS_NONE,
-									NULL,
-									"org.freedesktop.DBus",
-									"/org/freedesktop/DBus",
-									"org.freedesktop.DBus",
-									NULL,
-									&error);
-	
-	if ((proxy == NULL) && (error != NULL)) {
-		g_debug("Core error: %s\n", error->message);
-		g_error_free(error);
-		g_object_unref(connection);
-		return NULL;
-	}
-	
-	if (activatable) {
-		command = "ListActivatableNames";
-	} else {
-		command = "ListNames";
-	}
-	
-	error = NULL;
-	
-	services = g_dbus_proxy_call_sync(proxy,
-									command,
-									NULL,
-									0,
-									-1,
-									NULL,
-									&error);
-	
-	if ((services == NULL) && (error != NULL)) {
-		g_debug("Core error: %s\n", error->message);
-		g_error_free(error);
-		g_object_unref(proxy);
-		g_object_unref(connection);
-		return NULL;
-	}
-	
-	serviceshash = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, mmguicore_system_services_destroy_notify);
-	
-	g_variant_iter_init(&siterl1, services);
-	while ((snodel1 = g_variant_iter_next_value(&siterl1)) != NULL) {
-		g_variant_iter_init(&siterl2, snodel1);
-		while ((snodel2 = g_variant_iter_next_value(&siterl2)) != NULL) {
-			snamesize = 256;
-			servicename = g_strdup(g_variant_get_string(snodel2, &snamesize));
-			//g_printf("SERVICE (%s) %s\n", command, servicename);
-			g_hash_table_insert(serviceshash, (gpointer)servicename, (gpointer)servicename);
-			g_variant_unref(snodel2);
-		}
-		g_variant_unref(snodel1);
-    }
-	
-	g_variant_unref(services);
-	g_object_unref(proxy);
-	g_object_unref(connection);
-	
-	return serviceshash;
-} 
-
 static gint mmguicore_modules_sort(gconstpointer a, gconstpointer b)
 {
 	mmguimodule_t mod1, mod2;
@@ -409,7 +351,7 @@ static gboolean mmguicore_modules_cache_open(mmguicore_t mmguicore)
 	struct stat statbuf;
 	gchar **groups;
 	gsize numgroups;
-	gint i;
+	gint i, filever;
 	GError *error;
 	
 	if (mmguicore == NULL) return FALSE;
@@ -452,27 +394,31 @@ static gboolean mmguicore_modules_cache_open(mmguicore_t mmguicore)
 			cachetime = (time_t)g_key_file_get_uint64(mmguicore->cachekeyfile, MMGUICORE_FILE_ROOT_SECTION, MMGUICORE_FILE_TIMESTAMP, &error);
 			if (error == NULL) {
 				if (difftime(cachetime, mmguicore->updcachetime) != 0.0) {
-					groups = g_key_file_get_groups(mmguicore->cachekeyfile, &numgroups);
-					if ((groups != NULL) && (numgroups > 0)) {
-						for (i=0; i<numgroups; i++) {
-							error = NULL;
-							g_key_file_remove_group(mmguicore->cachekeyfile, groups[i], &error);
-							if (error != NULL) {
-								g_debug("Unable to remove cached module information: %s", error->message);
-								g_error_free(error);
-							}
-						}
-						g_strfreev(groups);
-					}
 					mmguicore->updatecache = TRUE;
 					g_debug("Local cache is outdated");
 				} else {
-					mmguicore->updatecache = FALSE;
-					g_debug("Local cache file is up to date");
+					/*Timestamp is up to date - check version*/
+					filever = g_key_file_get_integer(mmguicore->cachekeyfile, MMGUICORE_FILE_ROOT_SECTION, MMGUICORE_FILE_VERSION, &error);
+					if (error == NULL) {
+						if (filever >= MMGUICORE_CACHE_VER) {
+							/*Acceptable version of file*/
+							mmguicore->updatecache = FALSE;
+							g_debug("Local cache file is up to date");
+						} else {
+							/*Old version of file*/
+							mmguicore->updatecache = TRUE;
+							g_debug("Old version of file: %u", filever);
+						}
+					} else {
+						/*Unknown version of file*/
+						mmguicore->updatecache = TRUE;
+						g_debug("Local cache contains unreadable version identifier: %s", error->message);
+						g_error_free(error);
+					}
 				}
 			} else {
 				mmguicore->updatecache = TRUE;
-				g_debug("Local cache contain unreadable timestamp: %s", error->message);
+				g_debug("Local cache contains unreadable timestamp: %s", error->message);
 				g_error_free(error);
 			}
 		} else {
@@ -482,6 +428,22 @@ static gboolean mmguicore_modules_cache_open(mmguicore_t mmguicore)
 		}
 	}
 	
+	/*Cleanup file*/
+	if (mmguicore->updatecache) {
+		groups = g_key_file_get_groups(mmguicore->cachekeyfile, &numgroups);
+		if ((groups != NULL) && (numgroups > 0)) {
+			for (i=0; i<numgroups; i++) {
+				error = NULL;
+				g_key_file_remove_group(mmguicore->cachekeyfile, groups[i], &error);
+				if (error != NULL) {
+					g_debug("Unable to remove cached module information: %s", error->message);
+					g_error_free(error);
+				}
+			}
+			g_strfreev(groups);
+		}
+	}
+	
 	return TRUE;
 }
 
@@ -497,6 +459,8 @@ static gboolean mmguicore_modules_cache_close(mmguicore_t mmguicore)
 	if (mmguicore->updatecache) {
 		/*Save timestamp*/
 		g_key_file_set_int64(mmguicore->cachekeyfile, MMGUICORE_FILE_ROOT_SECTION, MMGUICORE_FILE_TIMESTAMP, (gint64)mmguicore->updcachetime);
+		/*Save version of file*/
+		g_key_file_set_integer(mmguicore->cachekeyfile, MMGUICORE_FILE_ROOT_SECTION, MMGUICORE_FILE_VERSION, MMGUICORE_CACHE_VER);
 		/*Write to file*/
 		error = NULL;
 		filedata = g_key_file_to_data(mmguicore->cachekeyfile, &datasize, &error);
@@ -530,6 +494,7 @@ static gboolean mmguicore_modules_cache_add_module(mmguicore_t mmguicore, mmguim
 	g_key_file_set_integer(mmguicore->cachekeyfile, module->filename, MMGUICORE_FILE_FUNCTIONS, module->functions);
 	g_key_file_set_string(mmguicore->cachekeyfile, module->filename, MMGUICORE_FILE_DESCRIPTION, module->description);
 	g_key_file_set_string(mmguicore->cachekeyfile, module->filename, MMGUICORE_FILE_SERVICENAME, module->servicename);
+	g_key_file_set_string(mmguicore->cachekeyfile, module->filename, MMGUICORE_FILE_SYSTEMDNAME, module->systemdname);
 	
 	return TRUE;
 }
@@ -656,7 +621,7 @@ static GSList *mmguicore_modules_cache_form_list(mmguicore_t mmguicore)
 				continue;
 			}
 			/*Module functions*/
-			modparams->functions = mmguicore_modules_cache_get_enumeration_value(mmguicore->cachekeyfile, groups[i], MMGUICORE_FILE_FUNCTIONS, MMGUI_MODULE_FUNCTION_BASIC, -1);
+			modparams->functions = mmguicore_modules_cache_get_enumeration_value(mmguicore->cachekeyfile, groups[i], MMGUICORE_FILE_FUNCTIONS, MMGUI_MODULE_FUNCTION_BASIC, MMGUI_MODULE_FUNCTION_POLKIT_PROTECTION, -1);
 			if (modparams->functions == -1) {
 				g_free(modparams);
 				continue;
@@ -671,6 +636,11 @@ static GSList *mmguicore_modules_cache_form_list(mmguicore_t mmguicore)
 				g_free(modparams);
 				continue;
 			}
+			/*Module systemd name*/
+			if (mmguicore_modules_cache_get_string_value(mmguicore->cachekeyfile, groups[i], MMGUICORE_FILE_SYSTEMDNAME, modparams->systemdname, sizeof(modparams->systemdname)) == NULL) {
+				g_free(modparams);
+				continue;
+			}
 			/*Module filename*/
 			strncpy(modparams->filename, groups[i], sizeof(modparams->filename));
 			/*Add to list*/
@@ -754,134 +724,151 @@ static GSList *mmguicore_modules_dir_form_list(mmguicore_t mmguicore, gboolean u
 	/*Close directory*/
 	g_dir_close(moddir);
 	
+	/*Save cache file*/
+	if ((mmguicore != NULL) && (updatecache)) {
+		mmguicore_modules_cache_close(mmguicore);
+	}
+	
 	return modules;
 }
 
-static guint mmguicore_modules_enum(mmguicore_t mmguicore)
+static gboolean mmguicore_modules_enumerate(mmguicore_t mmguicore)
 {
-	GHashTable *runningservices, *activatableservices;
-	GSList *modules;
 	mmguimodule_t modparams;
-	guint modnumber;
 	GSList *iterator;
-	//gchar shortname[256];
 	gsize shortnamelen;
+	gboolean mmavailable, cmavailable;
 	
+	if (mmguicore == NULL) return FALSE;
 	
-	if (mmguicore == NULL) return 0;
-	
-	/*Already running services*/
-	runningservices = mmguicore_system_services_enum(FALSE);
-	/*Services that can be activated*/
-	activatableservices = mmguicore_system_services_enum(TRUE);
-	
-	if ((runningservices == NULL) || (activatableservices == NULL)) {
-		g_debug("Unable to get lists of system services");
-		return 0;
-	}
-	
+	/*Form full modules list*/
 	if (mmguicore->updatecache) {
 		/*Got to get all available modules*/
-		modules = mmguicore_modules_dir_form_list(mmguicore, TRUE);
+		mmguicore->modules = mmguicore_modules_dir_form_list(mmguicore, TRUE);
 	} else {
 		/*Can use cache*/
-		modules = mmguicore_modules_cache_form_list(mmguicore);
+		mmguicore->modules = mmguicore_modules_cache_form_list(mmguicore);
 		/*Fallback if cache is broken*/
-		if (modules == NULL) {
+		if (mmguicore->modules == NULL) {
 			if ((mmguicore->cachefilename != NULL) && (mmguicore->cachekeyfile != NULL)) {
 				/*Cache can be updated*/
 				mmguicore->updatecache = TRUE;
-				modules = mmguicore_modules_dir_form_list(mmguicore, TRUE);
+				mmguicore->modules = mmguicore_modules_dir_form_list(mmguicore, TRUE);
 			} else {
 				/*Unable to update cache*/
-				modules = mmguicore_modules_dir_form_list(mmguicore, FALSE);
+				mmguicore->modules = mmguicore_modules_dir_form_list(mmguicore, FALSE);
 			}
 		}
 	}
 	
-	modnumber = 0;
+	/*We are going to check if at least one modem and connection management module is available*/
+	mmavailable = FALSE;
+	cmavailable = FALSE;
 	
 	/*Mark available modules*/
-	if (modules != NULL) {
-		for (iterator=modules; iterator; iterator=iterator->next) {
+	if (mmguicore->modules != NULL) {
+		for (iterator=mmguicore->modules; iterator; iterator=iterator->next) {
 			modparams = iterator->data;
-			/*Test if module is recommended*/
-			modparams->recommended = FALSE;
 			shortnamelen = sizeof(modparams->shortname);
-			if (mmguicore->options != NULL) {
-				if (mmguicore_modules_extract_name(modparams->filename, modparams->shortname, &shortnamelen) != NULL) {
+			if (mmguicore_modules_extract_name(modparams->filename, modparams->shortname, &shortnamelen) != NULL) {
+				/*Test service/file existence*/
+				modparams->applicable = FALSE;
+				modparams->activationtech = MMGUI_SVCMANGER_ACTIVATION_TECH_NA;
+				switch (modparams->requirement) {
+					case MMGUI_MODULE_REQUIREMENT_SERVICE:
+						modparams->applicable = mmgui_svcmanager_get_service_state(mmguicore->svcmanager, modparams->systemdname, modparams->servicename);
+						if (!modparams->applicable) {
+							modparams->activationtech = mmgui_svcmanager_get_service_activation_tech(mmguicore->svcmanager, modparams->systemdname, modparams->servicename);
+						}
+						break;
+					case MMGUI_MODULE_REQUIREMENT_FILE:
+						if (g_file_test(modparams->servicename, G_FILE_TEST_EXISTS)) {
+							modparams->applicable = TRUE;
+						}
+						break;
+					case MMGUI_MODULE_REQUIREMENT_NONE:
+						modparams->applicable = TRUE;
+						break;
+					default:
+						break;
+				}
+				/*Set module type available flag*/
+				if ((modparams->applicable) || (modparams->activationtech != MMGUI_SVCMANGER_ACTIVATION_TECH_NA)) {
 					switch (modparams->type) {
 						case MMGUI_MODULE_TYPE_MODEM_MANAGER:
-							if (mmguicore->options->mmmodule != NULL) {
-								if (strncmp(modparams->shortname, mmguicore->options->mmmodule, shortnamelen) == 0) {
-									modparams->recommended = TRUE;
-								}
-							}
+							mmavailable = TRUE;
 							break;
 						case MMGUI_MODULE_TYPE_CONNECTION_MANGER:
-							if (mmguicore->options->cmmodule != NULL) {
-								if (strncmp(modparams->shortname, mmguicore->options->cmmodule, shortnamelen) == 0) {
-									modparams->recommended = TRUE;
-								}
-							}
+							cmavailable = TRUE;
 							break;
 						default:
 							break;
 					}
-				}
+				}	
 			}
-			/*Test service/file existence*/
-			modparams->applicable = FALSE;
-			modparams->activatable = FALSE;
-			switch (modparams->requirement) {
-				case MMGUI_MODULE_REQUIREMENT_SERVICE:
-					if (g_hash_table_lookup(runningservices, modparams->servicename) != NULL) {
-						/*Service already active - no need to reactivate*/
-						modparams->applicable = TRUE;
-					}
-					if (g_hash_table_lookup(activatableservices, modparams->servicename) != NULL) {
-						/*Service can be activated*/
-						modparams->activatable = TRUE;
-						if ((modparams->recommended) && (!modparams->applicable)) {
-							/*Try to activate service if recommended*/
-							if (mmguicore_system_services_activate(modparams->servicename, NULL)) {
-								g_debug("DBus service started: %s (%s)\n", modparams->servicename, modparams->filename);
-								modparams->applicable = TRUE;
-							}
-						}
-					}
-					break;
-				case MMGUI_MODULE_REQUIREMENT_FILE:
-					if (g_file_test(modparams->servicename, G_FILE_TEST_EXISTS)) {
-						modparams->applicable = TRUE;
-						modparams->activatable = TRUE;
-					}
-					break;
-				case MMGUI_MODULE_REQUIREMENT_NONE:
-					modparams->applicable = TRUE;
-					modparams->activatable = TRUE;
-					break;
-				default:
-					break;
-			}
-			/*Increase modules count*/
-			modnumber++;
 		}
 		
 		/*Sort modules*/
-		modules = g_slist_sort(modules, mmguicore_modules_sort);
+		mmguicore->modules = g_slist_sort(mmguicore->modules, mmguicore_modules_sort);
 	}
 	
-	/*Free system services lists*/
-	g_hash_table_destroy(runningservices);
-	g_hash_table_destroy(activatableservices);
-	
-	/*Save modules list*/
-	mmguicore->modules = modules;
-	
-	return modnumber;
+	return mmavailable && cmavailable;
 } 
 
+static gboolean mmguicore_modules_prepare(mmguicore_t mmguicore)
+{
+	mmguimodule_t modparams;
+	GSList *iterator;
+	
+	if (mmguicore == NULL) return FALSE;
+	if (mmguicore->modules == NULL) return FALSE;
+	
+	for (iterator=mmguicore->modules; iterator; iterator=iterator->next) {
+		modparams = iterator->data;
+		/*Test if module is recommended*/
+		modparams->recommended = FALSE;
+		switch (modparams->type) {
+			case MMGUI_MODULE_TYPE_MODEM_MANAGER:
+				/*Set recommended flag*/
+				if (mmguicore->options != NULL) {
+					if (mmguicore->options->mmmodule != NULL) {
+						if (g_str_equal(modparams->shortname, mmguicore->options->mmmodule)) {
+							modparams->recommended = TRUE;
+						}
+					}
+				}
+			case MMGUI_MODULE_TYPE_CONNECTION_MANGER:
+				/*Set recommended flag*/
+				if (mmguicore->options != NULL) {
+					if (mmguicore->options->cmmodule != NULL) {
+						if (g_str_equal(modparams->shortname, mmguicore->options->cmmodule)) {
+							modparams->recommended = TRUE;
+						}
+					}
+				}
+				break;
+			default:
+				break;
+		}
+		/*Schedule activation if recommended*/
+		if ((modparams->recommended) && (!modparams->applicable) && (modparams->activationtech != MMGUI_SVCMANGER_ACTIVATION_TECH_NA) && (modparams->requirement == MMGUI_MODULE_REQUIREMENT_SERVICE)) {
+			mmgui_svcmanager_schedule_start_service(mmguicore->svcmanager, modparams->systemdname, modparams->servicename, modparams->shortname, mmguicore->options->enableservices);
+		}
+	}
+	
+	/*Initiate services activation*/
+	mmgui_svcmanager_start_services_activation(mmguicore->svcmanager);
+	
+	return TRUE;	
+}
+
+GSList *mmguicore_modules_get_list(mmguicore_t mmguicore)
+{
+	if (mmguicore == NULL) return NULL;
+	
+	return mmguicore->modules;
+}
+
 static gboolean mmguicore_modules_free(mmguicore_t mmguicore)
 {
 	if (mmguicore == NULL) return FALSE;
@@ -896,16 +883,33 @@ static gboolean mmguicore_modules_mm_open(mmguicore_t mmguicore, mmguimodule_t m
 {
 	gboolean openstatus; 
 	gchar *modulepath;
-	
+	gchar *polkitaction;
+		
 	if ((mmguicore == NULL) || (mmguimodule == NULL)) return FALSE;
 	
+	/*Wrong module type*/
 	if (mmguimodule->type == MMGUI_MODULE_TYPE_CONNECTION_MANGER) return FALSE;
 	
+	/*Authentication process*/
+	if (mmguimodule->functions & MMGUI_MODULE_FUNCTION_POLKIT_PROTECTION) {
+		openstatus = TRUE;
+		/*Autheticate user if needed*/
+		polkitaction = g_strdup_printf("ru.linuxonly.modem-manager-gui.manage-modem-%s", mmguimodule->shortname);
+		if (mmgui_polkit_action_needed(mmguicore->polkit, polkitaction, FALSE)) {
+			openstatus = mmgui_polkit_request_password(mmguicore->polkit, polkitaction);
+		}
+		g_free(polkitaction);
+		/*Exit if not authenticated*/
+		if (!openstatus) {
+			return FALSE;
+		}
+	}
+	/*Dynamic loader*/
 	modulepath = g_strconcat(RESOURCE_MODULES_DIR, mmguimodule->filename, NULL);
 	mmguicore->module = g_module_open(modulepath, G_MODULE_BIND_LAZY);
 	if (mmguicore->module != NULL) {
 		openstatus = TRUE;
-		//Module function pointers
+		/*Module function pointers*/
 		openstatus = openstatus && g_module_symbol(mmguicore->module, "mmgui_module_open", (gpointer *)&(mmguicore->open_func));
 		openstatus = openstatus && g_module_symbol(mmguicore->module, "mmgui_module_close", (gpointer *)&(mmguicore->close_func));
 		openstatus = openstatus && g_module_symbol(mmguicore->module, "mmgui_module_last_error", (gpointer *)&(mmguicore->last_error_func));
@@ -930,7 +934,7 @@ static gboolean mmguicore_modules_mm_open(mmguicore_t mmguicore, mmguimodule_t m
 		openstatus = openstatus && g_module_symbol(mmguicore->module, "mmgui_module_contacts_delete", (gpointer *)&(mmguicore->contacts_delete_func));
 		openstatus = openstatus && g_module_symbol(mmguicore->module, "mmgui_module_contacts_add", (gpointer *)&(mmguicore->contacts_add_func));
 		if (!openstatus) {
-			//Module function pointers
+			/*Module function pointers*/
 			mmguicore->open_func = NULL;
 			mmguicore->close_func = NULL;
 			mmguicore->last_error_func = NULL;
@@ -962,9 +966,19 @@ static gboolean mmguicore_modules_mm_open(mmguicore_t mmguicore, mmguimodule_t m
 	}
 	g_free(modulepath);
 	
+	/*Module preparation*/
 	if (openstatus) {
 		mmguicore->moduleptr = mmguimodule;
+		/*Open module*/
 		(mmguicore->open_func)(mmguicore);
+		/*Set module timeouts*/
+		if (mmguicore->options != NULL) {
+			mmguicore_modules_mm_set_timeouts(mmguicore, MMGUI_DEVICE_OPERATION_ENABLE, mmguicore->options->enabletimeout,
+														MMGUI_DEVICE_OPERATION_SEND_SMS, mmguicore->options->sendsmstimeout,
+														MMGUI_DEVICE_OPERATION_SEND_USSD, mmguicore->options->sendussdtimeout,
+														MMGUI_DEVICE_OPERATION_SCAN, mmguicore->options->scannetworkstimeout,
+														-1);
+		}
 		g_printf("Modem manager: %s\n", mmguimodule->description);
 	} else {
 		mmguicore->moduleptr = NULL;
@@ -978,11 +992,27 @@ static gboolean mmguicore_modules_cm_open(mmguicore_t mmguicore, mmguimodule_t m
 {
 	gboolean openstatus; 
 	gchar *modulepath;
+	gchar *polkitaction;
 		
 	if ((mmguicore == NULL) || (mmguimodule == NULL)) return FALSE;
 	
+	/*Wrong module type*/
 	if (mmguimodule->type == MMGUI_MODULE_TYPE_MODEM_MANAGER) return FALSE;
-	
+	/*Authentication process*/
+	if (mmguimodule->functions & MMGUI_MODULE_FUNCTION_POLKIT_PROTECTION) {
+		openstatus = TRUE;
+		/*Autheticate user if needed*/
+		polkitaction = g_strdup_printf("ru.linuxonly.modem-manager-gui.manage-network-%s", mmguimodule->shortname);
+		if (mmgui_polkit_action_needed(mmguicore->polkit, polkitaction, FALSE)) {
+			openstatus = mmgui_polkit_request_password(mmguicore->polkit, polkitaction);
+		}
+		g_free(polkitaction);
+		/*Exit if not authenticated*/
+		if (!openstatus) {
+			return FALSE;
+		}
+	}
+	/*Dynamic loader*/
 	modulepath = g_strconcat(RESOURCE_MODULES_DIR, mmguimodule->filename, NULL);
 	mmguicore->cmodule = g_module_open(modulepath, G_MODULE_BIND_LAZY);
 	if (mmguicore->cmodule != NULL) {
@@ -1138,69 +1168,6 @@ static gboolean mmguicore_modules_select(mmguicore_t mmguicore)
 	return mmfound;
 }
 
-GSList *mmguicore_modules_info(void)
-{
-	GHashTable *runningservices, *activatableservices;
-	GSList *moduleslist;
-	GSList *iterator;
-	mmguimodule_t module;
-	gsize shortnamelen;
-	
-	/*Already running services*/
-	runningservices = mmguicore_system_services_enum(FALSE);
-	/*Services that can be activated*/
-	activatableservices = mmguicore_system_services_enum(TRUE);
-	
-	if ((runningservices == NULL) || (activatableservices == NULL)) {
-		g_debug("Unable to get lists of system services");
-		return 0;
-	}
-	
-	/*Enumerate modules*/
-	moduleslist = mmguicore_modules_dir_form_list(NULL, FALSE);
-	
-	if (moduleslist != NULL) {
-		for (iterator=moduleslist; iterator; iterator=iterator->next) {
-			module = iterator->data;
-			/*Test service/file existence*/
-			module->applicable = FALSE;
-			module->activatable = FALSE;
-			switch (module->requirement) {
-				case MMGUI_MODULE_REQUIREMENT_SERVICE:
-					if (g_hash_table_lookup(activatableservices, module->servicename) != NULL) {
-						module->activatable = TRUE;
-					}
-					if (g_hash_table_lookup(runningservices, module->servicename) != NULL) {
-						module->applicable = TRUE;
-					}
-					break;
-				case MMGUI_MODULE_REQUIREMENT_FILE:
-					if (g_file_test(module->servicename, G_FILE_TEST_EXISTS)) {
-						module->activatable = TRUE;
-						module->applicable = TRUE;
-					}
-					break;
-				case MMGUI_MODULE_REQUIREMENT_NONE:
-					module->activatable = TRUE;
-					module->applicable = TRUE;
-					break;
-				default:
-					break;
-			}
-			shortnamelen = sizeof(module->shortname);
-			mmguicore_modules_extract_name(module->filename, module->shortname, &shortnamelen);
-		}
-		/*Sort modules*/
-		moduleslist = g_slist_sort(moduleslist, mmguicore_modules_sort);
-	}
-	
-	/*Free system services lists*/
-	g_hash_table_destroy(runningservices);
-	g_hash_table_destroy(activatableservices);
-	
-	return moduleslist;
-}
-
 void mmguicore_modules_mm_set_timeouts(mmguicore_t mmguicore, gint operation1, gint timeout1, ...)
 {
 	va_list args;
@@ -1316,7 +1283,7 @@ gboolean mmguicore_devices_enum(mmguicore_t mmguicore)
 		device = iterator->data;
 		g_debug("Device: %s, %s, %s [%u] [%s]\n", device->manufacturer, device->model, device->version, device->id, device->persistentid);
 	}
-		
+	
 	return TRUE;
 }
 
@@ -1624,6 +1591,14 @@ gboolean mmguicore_devices_get_registered(mmguicore_t mmguicore)
 	return (mmguicore->devices_state_func)(mmguicore, MMGUI_DEVICE_STATE_REQUEST_REGISTERED);
 }
 
+gboolean mmguicore_devices_get_prepared(mmguicore_t mmguicore)
+{
+	if (mmguicore == NULL) return FALSE;
+	if ((mmguicore->device == NULL) || (mmguicore->devices_state_func == NULL)) return FALSE;
+	
+	return (mmguicore->devices_state_func)(mmguicore, MMGUI_DEVICE_STATE_REQUEST_PREPARED);
+}
+
 gboolean mmguicore_devices_get_connected(mmguicore_t mmguicore)
 {
 	if (mmguicore == NULL) return FALSE;
@@ -2304,7 +2279,7 @@ gboolean mmguicore_interrupt_operation(mmguicore_t mmguicore)
 	return (mmguicore->interrupt_operation_func)(mmguicore);
 }
 
-mmguicore_t mmguicore_new(mmgui_event_ext_callback callback/*, mmgui_connections_sync_callback connsynccb*/, mmgui_core_options_t options, gpointer userdata)
+mmguicore_t mmguicore_init(mmgui_event_ext_callback callback, mmgui_core_options_t options, gpointer userdata)
 {
 	mmguicore_t mmguicore;
 		
@@ -2352,51 +2327,72 @@ mmguicore_t mmguicore_new(mmgui_event_ext_callback callback/*, mmgui_connections
 	mmguicore->device_connection_status_func = NULL;
 	mmguicore->device_connection_timestamp_func = NULL;
 	mmguicore->device_connection_disconnect_func = NULL;
-	//Devices
+	/*Work thread*/
+	mmguicore->workthread = NULL;
+	/*Devices*/
 	mmguicore->devices = NULL;
 	mmguicore->device = NULL;
-	//Internal callback
-	mmguicore->eventcb = mmguicore_event_callback;
-	//External callback
+	/*External callback*/
 	mmguicore->extcb = callback;
-	//Connections sync callback
-	//mmguicore->connsynccb = connsynccb;
-	//Traffic limits
-	//mmguicore->limits = limits;
 	/*Core options*/
 	mmguicore->options = options;
-	//User data pointer
+	/*User data pointer*/
 	mmguicore->userdata = userdata;
-	//Open netlink interface
-	mmguicore->netlink = mmgui_netlink_open();
+	/*Open polkit interface*/
+	mmguicore->polkit = mmgui_polkit_open();
+	/*Open service manager interface*/
+	mmguicore->svcmanager = mmgui_svcmanager_open(mmguicore->polkit, mmguicore_svcmanager_callback, mmguicore);
 	/*Open modules cache*/
 	mmguicore_modules_cache_open(mmguicore);
-	/*Enumerate available modules*/
-	if (!mmguicore_modules_enum(mmguicore)) {
+	/*Prepare recommended modules*/
+	if (!mmguicore_modules_enumerate(mmguicore)) {
+		/*Close service manager interface*/
+		mmgui_svcmanager_close(mmguicore->svcmanager);
+		/*Close polkit interface*/
+		mmgui_polkit_close(mmguicore->polkit);
+		/*Free resources*/		
 		g_free(mmguicore);
-		g_debug("No modules found\n");
 		return NULL;
 	}
+	
+	return mmguicore;
+}
+
+gboolean mmguicore_start(mmguicore_t mmguicore)
+{
+	if (mmguicore == NULL) return FALSE;
+	
+	return mmguicore_modules_prepare(mmguicore);		
+}
+
+static gboolean mmguicore_main(mmguicore_t mmguicore)
+{
+	if (mmguicore == NULL) return FALSE;
+		
+	/*Internal callback*/
+	mmguicore->eventcb = mmguicore_event_callback;
+	
 	/*Select module*/
 	if (!mmguicore_modules_select(mmguicore)) {
+		/*Close modules cache*/
 		mmguicore_modules_close(mmguicore);
 		mmguicore_modules_free(mmguicore);
+		/*Close service manager interface*/
+		mmgui_svcmanager_close(mmguicore->svcmanager);
+		/*Close polkit interface*/
+		mmgui_polkit_close(mmguicore->polkit);
+		/*Free resources*/
 		g_free(mmguicore);
-		return NULL;
-	}
-	/*Set module timeouts*/
-	if (options != NULL) {
-		mmguicore_modules_mm_set_timeouts(mmguicore, MMGUI_DEVICE_OPERATION_ENABLE, options->enabletimeout,
-													MMGUI_DEVICE_OPERATION_SEND_SMS, options->sendsmstimeout,
-													MMGUI_DEVICE_OPERATION_SEND_USSD, options->sendussdtimeout,
-													MMGUI_DEVICE_OPERATION_SCAN, options->scannetworkstimeout,
-													-1);
+		return FALSE;
 	}
 	
+	/*Open netlink interface*/
+	mmguicore->netlink = mmgui_netlink_open();
+	
 	/*New day time*/
-	mmguicore->newdaytime = mmgui_trafficdb_get_new_day_timesatmp(time(NULL));
+	mmguicore->newdaytime = mmgui_trafficdb_get_new_day_timesatmp(time(NULL), NULL, NULL);
 	
-	//Work thread
+	/*Work thread*/
 	if (pipe(mmguicore->workthreadctl) == 0) {
 		#if GLIB_CHECK_VERSION(2,32,0)
 			g_mutex_init(&mmguicore->workthreadmutex);
@@ -2415,7 +2411,7 @@ mmguicore_t mmguicore_new(mmgui_event_ext_callback callback/*, mmgui_connections
 		mmguicore->workthread = NULL;
 	}
 	
-	return mmguicore;
+	return TRUE;
 }
 
 void mmguicore_close(mmguicore_t mmguicore)
@@ -2424,13 +2420,12 @@ void mmguicore_close(mmguicore_t mmguicore)
 	
 	if (mmguicore == NULL) return;
 	
-	//Close opened device
+	/*Close opened device*/
 	mmguicore_devices_close(mmguicore);
 	
-	//Stop work thread
-	workthreadcmd = MMGUI_THREAD_STOP_CMD;
-	
 	if (mmguicore->workthread != NULL) {
+		//Stop work thread
+		workthreadcmd = MMGUI_THREAD_STOP_CMD;
 		//Lock mutex
 		#if GLIB_CHECK_VERSION(2,32,0)
 			g_mutex_lock(&mmguicore->workthreadmutex);
@@ -2454,12 +2449,14 @@ void mmguicore_close(mmguicore_t mmguicore)
 		}
 	}
 	
-	//Close netlink interface
+	/*Close netlink interface*/
 	mmgui_netlink_close(mmguicore->netlink);
-	//Destroy connections hash table
-	//g_hash_table_destroy(mmguicore->connections);
+	/*Close service manager interface*/
+	mmgui_svcmanager_close(mmguicore->svcmanager);
+	/*Close polkit interface*/
+	mmgui_polkit_close(mmguicore->polkit);
 	
-	mmguicore_modules_cache_close(mmguicore);
+	//mmguicore_modules_cache_close(mmguicore);
 	
 	//Close opened modules
 	mmguicore_modules_close(mmguicore);
@@ -2602,7 +2599,7 @@ static gpointer mmguicore_work_thread(gpointer data)
 		currenttime = time(NULL);
 		
 		if (mmguicore->device != NULL) {
-			if (abs((gint)difftime(ifstatetime, currenttime)) <= 5) {
+			if (abs((gint)difftime(ifstatetime, currenttime)) <= 15) {
 				//Save old values
 				g_debug("Requesting network interface state information\n");
 				oldstate = mmguicore->device->connected;
@@ -2687,9 +2684,16 @@ static gpointer mmguicore_work_thread(gpointer data)
 						if (event.type & MMGUI_NETLINK_INTERFACE_EVENT_TYPE_ADD) {
 							g_debug("Created network interface event\n");
 							if (mmguicore->device != NULL) {
-								if (!mmguicore->device->connected) {
+								if ((!mmguicore->device->connected) && (event.up) && (event.running)) {
+									/*PPP or Ethernet interface*/
 									g_debug("Created network interface event signal\n");
 									ifstatetime = time(NULL);
+								} else if ((mmguicore->device->connected) && (!event.up) && (!event.running)) {
+									/*Ethernet interface*/
+									if (g_str_equal(mmguicore->device->interface, event.ifname)) {
+										g_debug("Brought down network interface event signal\n");
+										ifstatetime = time(NULL);
+									}
 								}
 							}
 						}
@@ -2760,7 +2764,7 @@ static gpointer mmguicore_work_thread(gpointer data)
 			if (mmguicore->extcb != NULL) {
 				(mmguicore->extcb)(MMGUI_EVENT_SMS_NEW_DAY, mmguicore, NULL, mmguicore->userdata);
 			}
-			mmguicore->newdaytime = mmgui_trafficdb_get_new_day_timesatmp(currenttime);
+			mmguicore->newdaytime = mmgui_trafficdb_get_new_day_timesatmp(currenttime, NULL, NULL);
 		}
 		
 		//Unlock mutex after work finished
