@@ -25,6 +25,8 @@
 #include <net/if.h>
 
 #include "netlink.h"
+#include "polkit.h"
+#include "svcmanager.h"
 #include "smsdb.h"
 
 #define MMGUI_SPEED_VALUES_NUMBER    20
@@ -32,25 +34,27 @@
 #define MMGUI_THREAD_SLEEP_PERIOD    1 /*seconds*/
 
 enum _mmgui_event {
+	/*Device events*/
 	MMGUI_EVENT_DEVICE_ADDED = 0,
 	MMGUI_EVENT_DEVICE_REMOVED,
 	MMGUI_EVENT_DEVICE_OPENED,
 	MMGUI_EVENT_DEVICE_CLOSING,
 	MMGUI_EVENT_DEVICE_ENABLED_STATUS,
 	MMGUI_EVENT_DEVICE_BLOCKED_STATUS,
+	MMGUI_EVENT_DEVICE_PREPARED_STATUS,
 	MMGUI_EVENT_DEVICE_CONNECTION_STATUS,
-		
+	/*Messaging events*/
 	MMGUI_EVENT_SMS_LIST_READY,
 	MMGUI_EVENT_SMS_COMPLETED,
 	MMGUI_EVENT_SMS_SENT,
 	MMGUI_EVENT_SMS_REMOVED,
 	MMGUI_EVENT_SMS_NEW_DAY,
-	
+	/*Network events*/
 	MMGUI_EVENT_SIGNAL_LEVEL_CHANGE,
 	MMGUI_EVENT_NETWORK_MODE_CHANGE,
 	MMGUI_EVENT_NETWORK_REGISTRATION_CHANGE,
 	MMGUI_EVENT_LOCATION_CHANGE,
-	
+	/*Modem events*/
 	MMGUI_EVENT_MODEM_ENABLE_RESULT,
 	MMGUI_EVENT_SCAN_RESULT,
 	MMGUI_EVENT_USSD_RESULT,
@@ -58,9 +62,18 @@ enum _mmgui_event {
 	MMGUI_EVENT_TRAFFIC_LIMIT,
 	MMGUI_EVENT_TIME_LIMIT,
 	MMGUI_EVENT_UPDATE_CONNECTIONS_LIST,
-	
-	MMGUI_EVENT_EXTEND_CAPABILITIES
-		
+	/*Special-purpose events*/
+	MMGUI_EVENT_EXTEND_CAPABILITIES,
+	MMGUI_EVENT_SERVICE_ACTIVATION_STARTED,
+	MMGUI_EVENT_SERVICE_ACTIVATION_SERVICE_CHANGED,
+	MMGUI_EVENT_SERVICE_ACTIVATION_SERVICE_ACTIVATED,
+	MMGUI_EVENT_SERVICE_ACTIVATION_SERVICE_ERROR,
+	MMGUI_EVENT_SERVICE_ACTIVATION_FINISHED,
+	MMGUI_EVENT_SERVICE_ACTIVATION_AUTH_ERROR,
+	MMGUI_EVENT_SERVICE_ACTIVATION_STARTUP_ERROR,
+	MMGUI_EVENT_SERVICE_ACTIVATION_OTHER_ERROR,
+	/*System addressbooks export*/
+	MMGUI_EVENT_ADDRESS_BOOKS_EXPORT_FINISHED
 };
 
 //External event callback
@@ -70,21 +83,21 @@ typedef void (*mmgui_event_int_callback)(enum _mmgui_event event, gpointer mmgui
 
 enum _mmgui_module_priority {
 	//module priority levels
-	MMGUI_MODULE_PRIORITY_LOW         = 0,
-	MMGUI_MODULE_PRIORITY_NORMAL      = 1,
-	MMGUI_MODULE_PRIORITY_RECOMMENDED = 2
+	MMGUI_MODULE_PRIORITY_LOW                = 0,
+	MMGUI_MODULE_PRIORITY_NORMAL             = 1,
+	MMGUI_MODULE_PRIORITY_RECOMMENDED        = 2
 };
 
 enum _mmgui_module_type {
 	//module types
-	MMGUI_MODULE_TYPE_MODEM_MANAGER = 0x00,
-	MMGUI_MODULE_TYPE_CONNECTION_MANGER
+	MMGUI_MODULE_TYPE_MODEM_MANAGER          = 0,
+	MMGUI_MODULE_TYPE_CONNECTION_MANGER      = 1
 };
 
 enum _mmgui_module_functions {
 	//module functions bitmask
-	MMGUI_MODULE_FUNCTION_BASIC     = 0x00,
-	//MMGUI_MODULE_FUNCTION_VOICECALL = 0x01
+	MMGUI_MODULE_FUNCTION_BASIC              =      0,
+	MMGUI_MODULE_FUNCTION_POLKIT_PROTECTION  = 1 << 0
 };
 
 enum _mmgui_module_requirement {
@@ -95,16 +108,17 @@ enum _mmgui_module_requirement {
 };
 
 struct _mmguimodule {
-	//module description
+	/*Module description*/
 	guint identifier;
 	gboolean applicable;
-	gboolean activatable;
 	gboolean recommended;
-	enum _mmgui_module_type type;
-	enum _mmgui_module_requirement requirement;
-	enum _mmgui_module_priority priority;
-	enum _mmgui_module_functions functions;
+	gint type;
+	gint requirement;
+	gint priority;
+	gint functions;
+	gint activationtech;
 	gchar servicename[256];
+	gchar systemdname[256];
 	gchar description[256];
 	gchar filename[256];
 	gchar shortname[256];
@@ -258,7 +272,8 @@ enum _mmgui_device_state_request {
 	MMGUI_DEVICE_STATE_REQUEST_ENABLED = 0,
 	MMGUI_DEVICE_STATE_REQUEST_LOCKED,
 	MMGUI_DEVICE_STATE_REQUEST_REGISTERED,
-	MMGUI_DEVICE_STATE_REQUEST_CONNECTED
+	MMGUI_DEVICE_STATE_REQUEST_CONNECTED,
+	MMGUI_DEVICE_STATE_REQUEST_PREPARED
 };
 
 enum _mmgui_event_action {
@@ -315,6 +330,7 @@ struct _mmgui_core_options {
 	/*Preferred modules*/
 	gchar *mmmodule;
 	gchar *cmmodule;
+	gboolean enableservices;
 	/*Timeouts*/
 	gint enabletimeout;
 	gint sendsmstimeout;
@@ -345,6 +361,7 @@ struct _mmguidevice {
 	gboolean enabled;
 	gboolean blocked;
 	gboolean registered;
+	gboolean prepared;
 	//gboolean powered;
 	enum _mmgui_device_operation operation;
 	//Info
@@ -491,8 +508,12 @@ struct _mmguicore {
 	mmgui_core_options_t options;
 	//User data pointer
 	gpointer userdata;
-	//Netlink interface
+	/*Netlink interface*/
 	mmgui_netlink_t netlink;
+	/*Polkit interface*/
+	mmgui_polkit_t polkit;
+	/*Service manager interface*/
+	mmgui_svcmanager_t svcmanager;
 	/*New day time*/
 	time_t newdaytime;
 	//Work thread
@@ -511,7 +532,8 @@ typedef struct _mmguicore *mmguicore_t;
 
 /*Modules*/
 //void mmguicore_modules_list(mmguicore_t mmguicore, mmgui_modules_enum_callback callback);
-GSList *mmguicore_modules_info(void);
+//GSList *mmguicore_modules_info(void);
+GSList *mmguicore_modules_get_list(mmguicore_t mmguicore);
 void mmguicore_modules_mm_set_timeouts(mmguicore_t mmguicore, gint operation1, gint timeout1, ...);
 //Devices
 gboolean mmguicore_devices_enum(mmguicore_t mmguicore);
@@ -522,6 +544,7 @@ mmguidevice_t mmguicore_devices_get_current(mmguicore_t mmguicore);
 gboolean mmguicore_devices_get_enabled(mmguicore_t mmguicore);
 gboolean mmguicore_devices_get_locked(mmguicore_t mmguicore);
 gboolean mmguicore_devices_get_registered(mmguicore_t mmguicore);
+gboolean mmguicore_devices_get_prepared(mmguicore_t mmguicore);
 gboolean mmguicore_devices_get_connected(mmguicore_t mmguicore);
 gboolean mmguicore_devices_update_device_state(mmguicore_t mmguicore);
 const gchar *mmguicore_devices_get_identifier(mmguicore_t mmguicore);
@@ -568,7 +591,9 @@ GSList *mmguicore_get_connections_changes(mmguicore_t mmguicore);
 gchar *mmguicore_get_last_error(mmguicore_t mmguicore);
 gchar *mmguicore_get_last_connection_error(mmguicore_t mmguicore);
 gboolean mmguicore_interrupt_operation(mmguicore_t mmguicore);
-mmguicore_t mmguicore_new(mmgui_event_ext_callback callback/*, mmgui_connections_sync_callback connsynccb*/, mmgui_core_options_t options, gpointer userdata);
+mmguicore_t mmguicore_init(mmgui_event_ext_callback callback, mmgui_core_options_t options, gpointer userdata);
+gboolean mmguicore_start(mmguicore_t mmguicore);
+//void mmguicore_start(mmguicore_t mmguicore);
 void mmguicore_close(mmguicore_t mmguicore);
 
 #endif /* __MMGUICORE_H__ */
