@@ -32,6 +32,7 @@
 #include "../encoding.h"
 
 #define MMGUI_MODULE_SERVICE_NAME  "org.freedesktop.ModemManager"
+#define MMGUI_MODULE_SYSTEMD_NAME  "modem-manager.service"
 #define MMGUI_MODULE_IDENTIFIER    60
 #define MMGUI_MODULE_DESCRIPTION   "Modem Manager <= 0.6.0/Wader"
 
@@ -644,6 +645,7 @@ G_MODULE_EXPORT gboolean mmgui_module_init(mmguimodule_t module)
 	module->identifier = MMGUI_MODULE_IDENTIFIER;
 	module->functions = MMGUI_MODULE_FUNCTION_BASIC;
 	g_snprintf(module->servicename, sizeof(module->servicename), MMGUI_MODULE_SERVICE_NAME);
+	g_snprintf(module->systemdname, sizeof(module->systemdname), MMGUI_MODULE_SYSTEMD_NAME);
 	g_snprintf(module->description, sizeof(module->description), MMGUI_MODULE_DESCRIPTION);
 	
 	return TRUE;
@@ -961,6 +963,10 @@ G_MODULE_EXPORT gboolean mmgui_module_devices_state(gpointer mmguicore, enum _mm
 				res = FALSE;
 			}
 			break;
+		case MMGUI_DEVICE_STATE_REQUEST_PREPARED:
+			/*Not clear what state property means - just return TRUE*/
+			res = TRUE;
+			break;
 		default:
 			res = FALSE;
 			break;
@@ -2055,50 +2061,68 @@ static mmgui_sms_message_t mmgui_module_sms_retrieve(mmguicore_t mmguicore, GVar
 	index = 0;
 	
 	message = mmgui_smsdb_message_create();
-	//Sender number
+	
+	/*Sender number*/
 	value  = g_variant_lookup_value(messagev, "number", G_VARIANT_TYPE_STRING);
 	if (value != NULL) {
 		strlength = 256;
 		valuestr = g_variant_get_string(value, &strlength);
-		if (moduledata->needsmspolling) {
-			/*Old ModemManager versions tend to not bcd-decode sender numbers, doing it*/
-			decstr = (gchar *)bcd_to_utf8_ascii_part((const guchar *)valuestr, strlength, &declength);
-			if (decstr != NULL) {
-				mmgui_smsdb_message_set_number(message, decstr);
-				g_debug("SMS number: %s\n", decstr);
-				g_free(decstr);
+		if ((valuestr != NULL) && (valuestr[0] != '\0')) {
+			if (moduledata->needsmspolling) {
+				/*Old ModemManager versions tend to not bcd-decode sender numbers, doing it*/
+				decstr = (gchar *)bcd_to_utf8_ascii_part((const guchar *)valuestr, strlength, &declength);
+				if (decstr != NULL) {
+					mmgui_smsdb_message_set_number(message, decstr);
+					g_debug("SMS number: %s\n", decstr);
+					g_free(decstr);
+				} else {
+					mmgui_smsdb_message_set_number(message, valuestr);
+					g_debug("SMS number: %s\n", valuestr);
+				}
 			} else {
 				mmgui_smsdb_message_set_number(message, valuestr);
 				g_debug("SMS number: %s\n", valuestr);
 			}
-			
+		} else {
+			mmgui_smsdb_message_set_number(message, _("Unknown"));
 		}
 		g_variant_unref(value);
 	} else {
-		mmgui_smsdb_message_set_number(message, "--");
+		mmgui_smsdb_message_set_number(message, _("Unknown"));
 	}
-	//Service center number
+	
+	/*Service center number*/
 	value = g_variant_lookup_value(messagev, "smsc", G_VARIANT_TYPE_STRING);
 	if (value != NULL) {
 		strlength = 256;
 		valuestr = g_variant_get_string(value, &strlength);
-		mmgui_smsdb_message_set_service_number(message, valuestr);
-		g_debug("SMS service center number: %s\n", valuestr);
+		if ((valuestr != NULL) && (valuestr[0] != '\0')) {
+			mmgui_smsdb_message_set_service_number(message, valuestr);
+			g_debug("SMS service center number: %s\n", valuestr);
+		} else {
+			mmgui_smsdb_message_set_service_number(message, _("Unknown"));
+		}
 		g_variant_unref(value);
 	} else {
-		mmgui_smsdb_message_set_service_number(message, "--");
+		mmgui_smsdb_message_set_service_number(message, _("Unknown"));
 	}
-	//Decoded message text
+	
+	/*Try to get decoded message text first*/
 	gottext = FALSE;
 	value = g_variant_lookup_value(messagev, "text", G_VARIANT_TYPE_STRING);
 	if (value != NULL) {
 		strlength = 256*160;
 		valuestr = g_variant_get_string(value, &strlength);
-		g_debug("SMS text: %s\n", valuestr);
-		mmgui_smsdb_message_set_text(message, valuestr, FALSE);
-		gottext = TRUE;
+		if ((valuestr != NULL) && (valuestr[0] != '\0')) {
+			mmgui_smsdb_message_set_text(message, valuestr, FALSE);
+			gottext = TRUE;
+			g_debug("SMS text: %s\n", valuestr);
+		}
 		g_variant_unref(value);
-	} else {
+	}
+	
+	/*If there is no text (message isn't decoded), try to get binary data*/
+	if (!gottext) {
 		value = g_variant_lookup_value(messagev, "data", G_VARIANT_TYPE_BYTESTRING);
 		if (value != NULL) {
 			strlength = g_variant_get_size(value);
@@ -2111,24 +2135,28 @@ static mmgui_sms_message_t mmgui_module_sms_retrieve(mmguicore_t mmguicore, GVar
 			g_variant_unref(value);
 		}
 	}
-	//Message timestamp (ModemManager uses string, Wader uses double)
+	
+	/*Message timestamp (ModemManager uses string, Wader uses double)*/
 	if (moduledata->service == MODULE_INT_SERVICE_MODEM_MANAGER) {
 		value = g_variant_lookup_value(messagev, "timestamp", G_VARIANT_TYPE_STRING);
 		if (value != NULL) {
 			strlength = 256;
 			valuestr = g_variant_get_string(value, &strlength);
-			mmgui_smsdb_message_set_timestamp(message, mmgui_module_str_to_time(valuestr));
-			//g_debug("SMS timestamp: %ul\n", message->timestamp);
+			if ((valuestr != NULL) && (valuestr[0] != '\0')) {
+				mmgui_smsdb_message_set_timestamp(message, mmgui_module_str_to_time(valuestr));
+				g_debug("SMS timestamp: %s", ctime((const time_t *)&message->timestamp));
+			}
 			g_variant_unref(value);
 		}
 	} else if (moduledata->service == MODULE_INT_SERVICE_WADER) {
 		value = g_variant_lookup_value(messagev, "timestamp", G_VARIANT_TYPE_DOUBLE);
 		if (value != NULL) {
 			mmgui_smsdb_message_set_timestamp(message, (time_t)g_variant_get_double(value));
-			//g_debug("SMS timestamp: %ul\n", message->timestamp);
+			g_debug("SMS timestamp: %s", ctime((const time_t *)&message->timestamp));
 			g_variant_unref(value);
 		}
 	}
+	
 	/*Message index (ModemManager uses unsigned integer, Wader uses signed one.)
 	  This is a critical parameter, so message will be skipped if index value unknown.*/
 	gotindex = FALSE;
@@ -2136,27 +2164,28 @@ static mmgui_sms_message_t mmgui_module_sms_retrieve(mmguicore_t mmguicore, GVar
 		value = g_variant_lookup_value(messagev, "index", G_VARIANT_TYPE_UINT32);
 		if (value != NULL) {
 			index = g_variant_get_uint32(value);
-			g_variant_unref(value);
+			mmgui_smsdb_message_set_identifier(message, index, FALSE);
+			g_debug("SMS index: %u\n", index);
 			gotindex = TRUE;
+			g_variant_unref(value);
 		}
 	} else if (moduledata->service == MODULE_INT_SERVICE_WADER) {
 		value = g_variant_lookup_value(messagev, "index", G_VARIANT_TYPE_INT32);
 		if (value != NULL) {
 			index = (guint)g_variant_get_int32(value);
-			g_variant_unref(value);
+			mmgui_smsdb_message_set_identifier(message, index, FALSE);
+			g_debug("SMS index: %u\n", index);
 			gotindex = TRUE;
+			g_variant_unref(value);
 		}
 	}
 	
 	if ((!gotindex) || (!gottext)) {
-		//Message identifier unknown or no text - skip it
+		/*Message identifier unknown or no text - skip it*/
 		mmgui_smsdb_message_free(message);
 		return NULL;
 	}
 	
-	//Return message
-	mmgui_smsdb_message_set_identifier(message, index, FALSE);
-	g_debug("SMS index: %u\n", index);
 	return message;
 }
 
