@@ -31,11 +31,13 @@
 #include "../mmguicore.h"
 
 #define MMGUI_MODULE_SERVICE_NAME  "org.freedesktop.NetworkManager"
+#define MMGUI_MODULE_SYSTEMD_NAME  "NetworkManager.service"
 #define MMGUI_MODULE_IDENTIFIER    90
 #define MMGUI_MODULE_DESCRIPTION   "Network Manager >= 0.9.0"
 
 //Internal definitions
-#define MODULE_INT_PPPD_LOCK_FILE_PATH          "/var/run/%s.pid"
+#define MODULE_INT_NM_TIMESTAMPS_FILE_PATH     "/var/lib/NetworkManager/timestamps"
+#define MODULE_INT_NM_TIMESTAMPS_FILE_SECTION  "timestamps"
 
 //Internal enumerations
 //Modem state internal flags
@@ -130,6 +132,7 @@ G_MODULE_EXPORT gboolean mmgui_module_init(mmguimodule_t module)
 	module->identifier = MMGUI_MODULE_IDENTIFIER;
 	module->functions = MMGUI_MODULE_FUNCTION_BASIC;
 	g_snprintf(module->servicename, sizeof(module->servicename), MMGUI_MODULE_SERVICE_NAME);
+	g_snprintf(module->systemdname, sizeof(module->systemdname), MMGUI_MODULE_SYSTEMD_NAME);
 	g_snprintf(module->description, sizeof(module->description), MMGUI_MODULE_DESCRIPTION);
 	
 	return TRUE;
@@ -419,14 +422,13 @@ G_MODULE_EXPORT guint64 mmgui_module_device_connection_timestamp(gpointer mmguic
 	mmguicore_t mmguicorelc;
 	moduledata_t moduledata;
 	GError *error;
-	GDBusProxy *nmdevproxy;
-	GVariant *devproperty;
-	const gchar *devinterface;
-	guint devstate;
-	gsize strlength;
-	gchar lockfilepath[128];
-	guint64 timestamp;
-	struct stat statbuf;
+	GDBusProxy *devproxy;
+	GDBusProxy *connproxy;
+	GVariant *property;
+	guint64 curts, realts;
+	const gchar *connpath;
+	const gchar *connuuid;
+	GKeyFile *tsfile;
 	
 	if (mmguicore == NULL) return FALSE;
 	mmguicorelc = (mmguicore_t)mmguicore;
@@ -438,12 +440,12 @@ G_MODULE_EXPORT guint64 mmgui_module_device_connection_timestamp(gpointer mmguic
 	if (moduledata->nmdevpath == NULL) return FALSE;
 	
 	/*Get current timestamp*/
-	timestamp = (guint64)time(NULL);
+	curts = (guint64)time(NULL);
 	
 	error = NULL;
 	
 	/*Device proxy*/
-	nmdevproxy = g_dbus_proxy_new_sync(moduledata->connection,
+	devproxy = g_dbus_proxy_new_sync(moduledata->connection,
 										G_DBUS_PROXY_FLAGS_NONE,
 										NULL,
 										"org.freedesktop.NetworkManager",
@@ -452,30 +454,72 @@ G_MODULE_EXPORT guint64 mmgui_module_device_connection_timestamp(gpointer mmguic
 										NULL,
 										&error);
 	
-	if ((nmdevproxy != NULL) && (error == NULL)) {
-		devproperty = g_dbus_proxy_get_cached_property(nmdevproxy, "State");
-		devstate = g_variant_get_uint32(devproperty);
-		g_variant_unref(devproperty);
-		/*If device connected, get interface name*/
-		if (devstate == MODULE_INT_DEVICE_STATE_ACTIVATED) {
-			devproperty = g_dbus_proxy_get_cached_property(nmdevproxy, "IpInterface");
-			if (devproperty != NULL) {
-				strlength = 256;
-				devinterface = g_variant_get_string(devproperty, &strlength);
-				/*Form lock file path*/
-				memset(lockfilepath, 0, sizeof(lockfilepath));
-				g_snprintf(lockfilepath, sizeof(lockfilepath), MODULE_INT_PPPD_LOCK_FILE_PATH, devinterface);
-				/*Get lock file modification timestamp*/
-				if (stat(lockfilepath, &statbuf) == 0) {
-					timestamp = (guint64)statbuf.st_mtime;
-				}
-				g_variant_unref(devproperty);
-			}
-		} 
-		g_object_unref(nmdevproxy);
+	if ((devproxy == NULL) && (error != NULL)) {
+		mmgui_module_handle_error_message(mmguicorelc, error);
+		g_error_free(error);
+		return curts;
 	}
 	
-	return timestamp;
+	/*Get path to Active Connection interface*/
+	property = g_dbus_proxy_get_cached_property(devproxy, "ActiveConnection");
+	connpath = g_variant_get_string(property, NULL);
+	
+	/*Active Connection proxy*/
+	connproxy = g_dbus_proxy_new_sync(moduledata->connection,
+										G_DBUS_PROXY_FLAGS_NONE,
+										NULL,
+										"org.freedesktop.NetworkManager",
+										connpath,
+										"org.freedesktop.NetworkManager.Connection.Active",
+										NULL,
+										&error);
+	
+	if ((connproxy == NULL) && (error != NULL)) {
+		mmgui_module_handle_error_message(mmguicorelc, error);
+		g_error_free(error);
+		g_variant_unref(property);
+		g_object_unref(devproxy);
+		return curts;
+	}
+	
+	/*Free first proxy resources*/
+	g_variant_unref(property);
+	g_object_unref(devproxy);
+	
+	/*Get active connection UUID*/
+	property = g_dbus_proxy_get_cached_property(connproxy, "Uuid");
+	connuuid = g_variant_get_string(property, NULL);
+	
+	/*Retrieve timestamp from Network Manager timestamps ini file*/
+	tsfile = g_key_file_new();
+	
+	if (!g_key_file_load_from_file(tsfile, MODULE_INT_NM_TIMESTAMPS_FILE_PATH, G_KEY_FILE_NONE, &error)) {
+		mmgui_module_handle_error_message(mmguicorelc, error);
+		g_error_free(error);
+		g_key_file_free(tsfile);
+		g_variant_unref(property);
+		g_object_unref(connproxy);
+		return curts;
+	}
+	
+	/*File has only one section 'timestamps' with elements 'uuid=timestamp'*/
+	realts = g_key_file_get_uint64(tsfile, MODULE_INT_NM_TIMESTAMPS_FILE_SECTION, connuuid, &error);
+	
+	if ((realts == 0) && (error != NULL)) {
+		mmgui_module_handle_error_message(mmguicorelc, error);
+		g_error_free(error);
+		g_key_file_free(tsfile);
+		g_variant_unref(property);
+		g_object_unref(connproxy);
+		return curts;
+	}
+	
+	/*Free second proxy and ini file resources*/
+	g_key_file_free(tsfile);
+	g_variant_unref(property);
+	g_object_unref(connproxy);
+	
+	return realts;
 }
 
 G_MODULE_EXPORT gboolean mmgui_module_device_connection_disconnect(gpointer mmguicore)
