@@ -35,6 +35,7 @@
 #include "../mmguicore.h"
 
 #define MMGUI_MODULE_SERVICE_NAME  "/usr/sbin/pppd"
+#define MMGUI_MODULE_SYSTEMD_NAME  "NULL"
 #define MMGUI_MODULE_IDENTIFIER    245
 #define MMGUI_MODULE_DESCRIPTION   "pppd >= 2.4.5"
 
@@ -265,6 +266,7 @@ G_MODULE_EXPORT gboolean mmgui_module_init(mmguimodule_t module)
 	module->identifier = MMGUI_MODULE_IDENTIFIER;
 	module->functions = MMGUI_MODULE_FUNCTION_BASIC;
 	g_snprintf(module->servicename, sizeof(module->servicename), MMGUI_MODULE_SERVICE_NAME);
+	g_snprintf(module->systemdname, sizeof(module->systemdname), MMGUI_MODULE_SYSTEMD_NAME);
 	g_snprintf(module->description, sizeof(module->description), MMGUI_MODULE_DESCRIPTION);
 	
 	return TRUE;
