@@ -19,6 +19,7 @@
  
 #include <glib.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <unistd.h>
@@ -41,6 +42,18 @@
 
 #define MMGUI_NETLINK_INTERNAL_SEQUENCE_NUMBER 100000
 
+
+static gboolean mmgui_netlink_numeric_name(gchar *dirname);
+static gboolean mmgui_netlink_process_access(gchar *dirname, uid_t uid);
+static gboolean mmgui_netlink_socket_access(gchar *dirname, gchar *sockname, guint inode);
+static gchar *mmgui_netlink_process_name(gchar *dirname, gchar *appname, gsize appsize);
+static gboolean mmgui_netlink_get_process(guint inode, gchar *appname, gsize namesize, pid_t *apppid);
+static gboolean mmgui_netlink_hash_clear_foreach(gpointer key, gpointer value, gpointer user_data);
+struct sockaddr_nl *mmgui_netlink_get_connections_monitoring_socket_address(mmgui_netlink_t netlink);
+struct sockaddr_nl *mmgui_netlink_get_interfaces_monitoring_socket_address(mmgui_netlink_t netlink);
+static mmgui_netlink_connection_change_t mmgui_netlink_create_connection_change(mmgui_netlink_t netlink, guint event, guint inode);
+
+
 static gboolean mmgui_netlink_numeric_name(gchar *dirname)
 {
 	if ((dirname == NULL) || (!*dirname)) return FALSE;
@@ -49,10 +62,10 @@ static gboolean mmgui_netlink_numeric_name(gchar *dirname)
 		if (!isdigit(*dirname)) {
 			return FALSE;
 		} else {
-			*dirname++;
+			(void)*dirname++;
 		}
 	}
-	
+		
 	return TRUE;
 }
 
@@ -87,7 +100,7 @@ static gboolean mmgui_netlink_socket_access(gchar *dirname, gchar *sockname, gui
 		
 	memset(fullpath, 0, sizeof(fullpath));
 	
-	sprintf(fullpath, "/proc/%s/fd/%s", dirname, sockname);
+	snprintf(fullpath, sizeof(fullpath), "/proc/%s/fd/%s", dirname, sockname);
 	
 	if (stat(fullpath, &fdstat) == -1) {
 		return FALSE;
@@ -110,13 +123,13 @@ static gchar *mmgui_netlink_process_name(gchar *dirname, gchar *appname, gsize a
 	if ((appname == NULL) || (appsize == 0)) return NULL;
 	
 	memset(fpath, 0, sizeof(fpath));
-	sprintf(fpath, "/proc/%s/exe", dirname);
+	snprintf(fpath, sizeof(fpath), "/proc/%s/exe", dirname);
 	
 	linkchars = readlink(fpath, appname, appsize-1);
 	
 	if (linkchars == 0) {
 		memset(fpath, 0, sizeof(fpath));
-		sprintf(fpath, "/proc/%s/comm", dirname);
+		snprintf(fpath, sizeof(fpath), "/proc/%s/comm", dirname);
 		
 		fd = open(fpath, O_RDONLY);
 		if (fd != -1) {
@@ -146,20 +159,20 @@ static gboolean mmgui_netlink_get_process(guint inode, gchar *appname, gsize nam
 {
 	DIR *procdir, *fddir;
 	struct dirent *procde, *fdde;
-	char fdirpath[128], fdpath[128];
+	gchar fdirpath[128];
 	
 	if ((appname == NULL) || (namesize == 0) || (apppid == NULL)) return FALSE;
 	
 	procdir = opendir("/proc");
 	if (procdir != NULL) {
-		while (procde = readdir(procdir)) {
+		while ((procde = readdir(procdir))) {
 			if (mmgui_netlink_process_access(procde->d_name, getuid())) {
 				memset(fdirpath, 0, sizeof(fdirpath));
-				sprintf(fdirpath, "/proc/%s/fd", procde->d_name);
+				snprintf(fdirpath, sizeof(fdirpath), "/proc/%s/fd", procde->d_name);
 				//enumerate file descriptors
 				fddir = opendir(fdirpath);
 				if (fddir != NULL) {
-					while (fdde = readdir(fddir)) {
+					while ((fdde = readdir(fddir))) {
 						if (mmgui_netlink_socket_access(procde->d_name, fdde->d_name, inode)) {
 							//printf("%s:%s (%s)\n", procde->d_name, fdde->d_name, nlconnections_process_name(procde->d_name, appname, sizeof(appname)));
 							*apppid = atoi(procde->d_name);
@@ -237,15 +250,23 @@ void mmgui_netlink_hash_destroy(gpointer data)
 static gboolean mmgui_netlink_hash_clear_foreach(gpointer key, gpointer value, gpointer user_data)
 {
 	mmgui_netlink_connection_t connection;
-	time_t currenttime;
+	mmgui_netlink_t netlink;
+	mmgui_netlink_connection_change_t change;
 	
 	connection = (mmgui_netlink_connection_t)value;
-	currenttime = *(time_t *)user_data;
+	netlink = (mmgui_netlink_t)user_data;
 	
-	if (connection->updatetime == currenttime) {
+	if (connection->updatetime == netlink->currenttime) {
 		return FALSE;
 	} else {
-		//printf("Remove: %u\n", socket->inode);
+		if (netlink->changequeue != NULL) {
+			change = mmgui_netlink_create_connection_change(netlink, MMGUI_NETLINK_CONNECTION_EVENT_REMOVE, connection->inode);
+			if (change != NULL) {
+				g_async_queue_push(netlink->changequeue, change);
+			}								
+		}
+		
+		g_debug("Connection removed: inode %u\n", connection->inode);
 		return TRUE;
 	}
 }
@@ -254,20 +275,20 @@ gboolean mmgui_netlink_request_connections_list(mmgui_netlink_t netlink, guint f
 {
 	struct _mmgui_netlink_connection_info_request request;
 	gint status;
-	
-	if ((netlink == NULL) || ((!(family & AF_INET)) && (!(family & AF_INET6)))) return FALSE;
+		
+	if ((netlink == NULL) || ((family != AF_INET) && (family != AF_INET6))) return FALSE;
 	
 	memset(&request.msgheader, 0, sizeof(struct nlmsghdr));
 	
 	request.msgheader.nlmsg_len = sizeof(struct _mmgui_netlink_connection_info_request);
 	request.msgheader.nlmsg_type = TCPDIAG_GETSOCK;
 	request.msgheader.nlmsg_flags = NLM_F_REQUEST | NLM_F_ROOT;
-	request.msgheader.nlmsg_pid = getpid();
+	request.msgheader.nlmsg_pid = ((1 & 0xffc00000) << 22) | (getpid() & 0x3fffff);
 	request.msgheader.nlmsg_seq = 0;
 	
 	memset(&request.nlreq, 0, sizeof(struct inet_diag_req));
 	request.nlreq.idiag_family = family;
-	request.nlreq.idiag_states = ((1 << TCP_CLOSING + 1) - 1);
+	request.nlreq.idiag_states = ((1 << (TCP_CLOSING + 1)) - 1);
 	
 	request.msgheader.nlmsg_len = NLMSG_ALIGN(request.msgheader.nlmsg_len);
 	
@@ -282,20 +303,21 @@ gboolean mmgui_netlink_request_connections_list(mmgui_netlink_t netlink, guint f
 
 gboolean mmgui_netlink_read_connections_list(mmgui_netlink_t netlink, gchar *data, gsize datasize)
 {
-	time_t currenttime;
 	struct nlmsghdr *msgheader;
 	struct inet_diag_msg *entry;
 	mmgui_netlink_connection_t connection;
-	struct hostent *dsthost;
+	mmgui_netlink_connection_change_t change;
+	/*struct hostent *dsthost;*/
 	gchar srcbuf[INET6_ADDRSTRLEN];
 	gchar dstbuf[INET6_ADDRSTRLEN];
 	gchar appname[1024];
 	pid_t apppid;
+	gboolean needupdate;
 			
 	if ((netlink == NULL) || (data == NULL) || (datasize == 0)) return FALSE;
 	
 	//Get current time
-	currenttime = time(NULL);
+	netlink->currenttime = time(NULL);
 	
 	//Work with data
 	for (msgheader = (struct nlmsghdr *)data; NLMSG_OK(msgheader, (unsigned int)datasize); msgheader = NLMSG_NEXT(msgheader, datasize)) {
@@ -314,7 +336,7 @@ gboolean mmgui_netlink_read_connections_list(mmgui_netlink_t netlink, gchar *dat
 							connection->inode = entry->idiag_inode;
 							connection->family = entry->idiag_family;
 							connection->userid = entry->idiag_uid;
-							connection->updatetime = currenttime;
+							connection->updatetime = netlink->currenttime;
 							connection->dqueue = entry->idiag_rqueue + entry->idiag_wqueue;
 							connection->state = entry->idiag_state;
 							connection->srcport = ntohs(entry->id.idiag_sport);
@@ -330,16 +352,38 @@ gboolean mmgui_netlink_read_connections_list(mmgui_netlink_t netlink, gchar *dat
 								connection->dsthostname = g_strdup(connection->dstaddr);
 							/*}*/
 							g_hash_table_insert(netlink->connections, (gpointer)&connection->inode, connection);
+							/*Add change*/
+							if (netlink->changequeue != NULL) {
+								change = mmgui_netlink_create_connection_change(netlink, MMGUI_NETLINK_CONNECTION_EVENT_ADD, connection->inode);
+								if (change != NULL) {
+									g_async_queue_push(netlink->changequeue, change);
+								}								
+							}
 							g_debug("Connection added: inode %u\n", entry->idiag_inode);
 						}
 					} else {
 						//Update connection information (state, buffers fill, time)
 						connection = g_hash_table_lookup(netlink->connections, (gconstpointer)&entry->idiag_inode);
 						if (connection != NULL) {
-							connection->updatetime = currenttime;
-							connection->dqueue = entry->idiag_rqueue + entry->idiag_wqueue;
-							connection->state = entry->idiag_state;
-							g_debug("Connection updated: inode %u\n", entry->idiag_inode);
+							connection->updatetime = netlink->currenttime;
+							needupdate = FALSE;
+							if (connection->dqueue != (entry->idiag_rqueue + entry->idiag_wqueue)) {
+								connection->dqueue = entry->idiag_rqueue + entry->idiag_wqueue;
+								needupdate = TRUE;
+							}
+							if (connection->state != entry->idiag_state) {
+								connection->state = entry->idiag_state;
+								needupdate = TRUE;
+							}
+							if (needupdate) {
+								if (netlink->changequeue != NULL) {
+									change = mmgui_netlink_create_connection_change(netlink, MMGUI_NETLINK_CONNECTION_EVENT_MODIFY, connection->inode);
+									if (change != NULL) {
+										g_async_queue_push(netlink->changequeue, change);
+									}								
+								}
+								g_debug("Connection updated: inode %u\n", entry->idiag_inode);
+							}
 						}	
 					}
 				}
@@ -348,8 +392,8 @@ gboolean mmgui_netlink_read_connections_list(mmgui_netlink_t netlink, gchar *dat
 	}
 	
 	//Remove connections that disappear
-	g_hash_table_foreach_remove(netlink->connections, mmgui_netlink_hash_clear_foreach, &currenttime);
-	
+	g_hash_table_foreach_remove(netlink->connections, mmgui_netlink_hash_clear_foreach, netlink);
+		
 	return TRUE;
 }
 
@@ -372,7 +416,7 @@ gboolean mmgui_netlink_request_interface_statistics(mmgui_netlink_t netlink, gch
 	request.msgheader.nlmsg_type = RTM_GETLINK;
 	request.msgheader.nlmsg_flags = NLM_F_REQUEST;
 	request.msgheader.nlmsg_seq = MMGUI_NETLINK_INTERNAL_SEQUENCE_NUMBER;
-	request.msgheader.nlmsg_pid = getpid();
+	request.msgheader.nlmsg_pid = ((2 & 0xffc00000) << 22) | (getpid() & 0x3fffff);
 	
 	request.ifinfo.ifi_family = AF_UNSPEC;
 	request.ifinfo.ifi_index = ifindex;
@@ -505,11 +549,174 @@ struct sockaddr_nl *mmgui_netlink_get_interfaces_monitoring_socket_address(mmgui
 	return &(netlink->intaddr);
 }
 
-GHashTable *mmgui_netlink_get_connections_list(mmgui_netlink_t netlink)
+static mmgui_netlink_connection_change_t mmgui_netlink_create_connection_change(mmgui_netlink_t netlink, guint event, guint inode)
 {
+	mmgui_netlink_connection_change_t change;
+	mmgui_netlink_connection_t connection;
+	
 	if (netlink == NULL) return NULL;
 	
-	return netlink->connections;
+	change = NULL;
+	
+	switch (event) {
+		case MMGUI_NETLINK_CONNECTION_EVENT_ADD:
+			connection = g_hash_table_lookup(netlink->connections, &inode);
+			if (connection != NULL) {
+				change = g_new0(struct _mmgui_netlink_connection_change, 1);
+				change->inode = inode;
+				change->event = event;
+				change->data.connection = g_new0(struct _mmgui_netlink_connection, 1);
+				change->data.connection->state = connection->state;
+				change->data.connection->family = connection->family;
+				change->data.connection->srcport = connection->srcport;
+				change->data.connection->dqueue = connection->dqueue;
+				change->data.connection->inode = connection->inode;
+				change->data.connection->userid = connection->userid;
+				change->data.connection->apppid = connection->apppid;
+				change->data.connection->appname = g_strdup(connection->appname);
+				change->data.connection->dsthostname = g_strdup(connection->dsthostname);
+				strncpy(change->data.connection->srcaddr, connection->srcaddr, sizeof(change->data.connection->srcaddr));
+				strncpy(change->data.connection->dstaddr, connection->dstaddr, sizeof(change->data.connection->dstaddr));
+			}
+			break;
+		case MMGUI_NETLINK_CONNECTION_EVENT_REMOVE:
+			change = g_new0(struct _mmgui_netlink_connection_change, 1);
+			change->inode = inode;
+			change->event = event;
+			change->data.connection = NULL;
+			break;
+		case MMGUI_NETLINK_CONNECTION_EVENT_MODIFY:
+			connection = g_hash_table_lookup(netlink->connections, &inode);
+			if (connection != NULL) {
+				change = g_new0(struct _mmgui_netlink_connection_change, 1);
+				change->inode = inode;
+				change->event = event;
+				change->data.params = g_new0(struct _mmgui_netlink_connection_changed_params, 1);
+				change->data.params->state = connection->state;
+				change->data.params->dqueue = connection->dqueue;
+			}
+			break;
+		 default:
+			break;
+	}
+	
+	return change;
+}
+
+void mmgui_netlink_free_connection_change(mmgui_netlink_connection_change_t change)
+{
+	if (change == NULL) return;
+	
+	switch (change->event) {
+		case MMGUI_NETLINK_CONNECTION_EVENT_ADD:
+			if (change->data.connection != NULL) {
+				if (change->data.connection->appname != NULL) {
+					g_free(change->data.connection->appname);
+				}
+				if (change->data.connection->dsthostname != NULL) {
+					g_free(change->data.connection->dsthostname);
+				}
+				g_free(change->data.connection);
+			}
+			g_free(change);
+			break;
+		case MMGUI_NETLINK_CONNECTION_EVENT_REMOVE:
+			g_free(change);
+			break;
+		case MMGUI_NETLINK_CONNECTION_EVENT_MODIFY:
+			if (change->data.params != NULL) {
+				g_free(change->data.params);				
+			}
+			g_free(change);
+			break;
+		 default:
+			break;
+	}
+}
+
+void mmgui_netlink_free_connection(mmgui_netlink_connection_t connection)
+{
+	if (connection == NULL) return;
+	
+	if (connection->appname != NULL) {
+		g_free(connection->appname);
+	}
+	
+	if (connection->dsthostname != NULL) {
+		g_free(connection->dsthostname);
+	}
+	
+	g_free(connection);
+}
+
+GSList *mmgui_netlink_open_interactive_connections_list(mmgui_netlink_t netlink)
+{
+	GSList *connections;
+	GHashTableIter iter;
+	gpointer key, value;
+	mmgui_netlink_connection_t srcconn, dstconn;
+	
+	if (netlink == NULL) return NULL;
+	
+	connections = NULL;
+	
+	g_hash_table_iter_init(&iter, netlink->connections);
+	while (g_hash_table_iter_next(&iter, &key, &value)) {
+		srcconn = (mmgui_netlink_connection_t)value;
+		if (srcconn != NULL) {
+			dstconn = g_new(struct _mmgui_netlink_connection, 1);
+			dstconn->inode = srcconn->inode;
+			dstconn->family = srcconn->family;
+			dstconn->userid = srcconn->userid;
+			dstconn->updatetime = srcconn->updatetime;
+			dstconn->dqueue = srcconn->dqueue;
+			dstconn->state = srcconn->state;
+			dstconn->srcport = srcconn->srcport;
+			strncpy(dstconn->srcaddr, srcconn->srcaddr, sizeof(dstconn->srcaddr));
+			strncpy(dstconn->dstaddr, srcconn->dstaddr, sizeof(dstconn->dstaddr));
+			dstconn->appname = g_strdup(srcconn->appname);
+			dstconn->apppid = srcconn->apppid;
+			dstconn->dsthostname = g_strdup(srcconn->dsthostname);
+			connections = g_slist_prepend(connections, dstconn);
+		}
+	}
+	if (connections != NULL) {
+		connections = g_slist_reverse(connections);
+	}
+	
+	if (netlink->changequeue == NULL) {
+		netlink->changequeue = g_async_queue_new_full((GDestroyNotify)mmgui_netlink_free_connection_change);
+	}
+	
+	return connections;
+}
+
+void mmgui_netlink_close_interactive_connections_list(mmgui_netlink_t netlink)
+{
+	if (netlink->changequeue != NULL) {
+		g_async_queue_unref(netlink->changequeue);
+		netlink->changequeue = NULL;
+	}
+}
+
+GSList *mmgui_netlink_get_connections_changes(mmgui_netlink_t netlink)
+{
+	GSList *changes;
+	mmgui_netlink_connection_change_t change;
+	
+	if (netlink->changequeue == NULL) return NULL;
+	
+	changes = NULL;
+	
+	while ((change = g_async_queue_try_pop(netlink->changequeue)) != NULL) {
+		changes = g_slist_prepend(changes, change);
+	}
+	
+	if (changes != NULL) {
+		changes = g_slist_reverse(changes);
+	}
+	
+	return changes;
 }
 
 void mmgui_netlink_close(mmgui_netlink_t netlink)
@@ -519,6 +726,9 @@ void mmgui_netlink_close(mmgui_netlink_t netlink)
 	if (netlink->connsocketfd != -1) {
 		close(netlink->connsocketfd);
 		g_hash_table_destroy(netlink->connections);
+		if (netlink->changequeue != NULL) {
+			g_async_queue_unref(netlink->changequeue);
+		}
 	}
 	
 	if (netlink->intsocketfd != -1) {
@@ -543,11 +753,11 @@ mmgui_netlink_t mmgui_netlink_open(void)
 	if (netlink->connsocketfd != -1) {
 		memset(&netlink->connaddr, 0, sizeof(struct sockaddr_nl));
 		netlink->connaddr.nl_family = AF_NETLINK;
-		netlink->connaddr.nl_pid = getpid();
+		netlink->connaddr.nl_pid = ((1 & 0xffc00000) << 22) | (getpid() & 0x3fffff);
 		netlink->connaddr.nl_groups = 0;
 		
 		netlink->userid = getuid();
-		
+		netlink->changequeue = NULL;
 		netlink->connections = g_hash_table_new_full(g_int_hash, g_int_equal, NULL, (GDestroyNotify)mmgui_netlink_hash_destroy);
 	} else {
 		netlink->connections = NULL;
@@ -560,7 +770,7 @@ mmgui_netlink_t mmgui_netlink_open(void)
 		memset(&netlink->intaddr, 0, sizeof(netlink->intaddr));
 		netlink->intaddr.nl_family = AF_NETLINK;
 		netlink->intaddr.nl_groups = RTMGRP_LINK;
-		netlink->intaddr.nl_pid = getpid();
+		netlink->intaddr.nl_pid = ((2 & 0xffc00000) << 22) | (getpid() & 0x3fffff);
 		
 		if (bind(netlink->intsocketfd, (struct sockaddr *)&(netlink->intaddr), sizeof(netlink->intaddr)) == -1) {
 			g_debug("Failed to bind network interfaces monitoring netlink socket\n");
