@@ -42,7 +42,7 @@ mmgui_notifications_t mmgui_notifications_new(mmgui_libpaths_cache_t libcache)
 	notifications->notifymodule = NULL;
 	
 	//Open module
-	notifications->notifymodule = g_module_open(mmgui_libpaths_cache_get_library_name(libcache, "libnotify"), G_MODULE_BIND_LAZY);
+	notifications->notifymodule = g_module_open(mmgui_libpaths_cache_get_library_full_path(libcache, "libnotify"), G_MODULE_BIND_LAZY);
 	
 	if (notifications->notifymodule != NULL) {
 		libopened = TRUE;
@@ -102,7 +102,7 @@ mmgui_notifications_t mmgui_notifications_new(mmgui_libpaths_cache_t libcache)
 	notifications->cacontext = NULL;
 	
 	//Open module
-	notifications->canberramodule = g_module_open(mmgui_libpaths_cache_get_library_name(libcache, "libcanberra"), G_MODULE_BIND_LAZY);
+	notifications->canberramodule = g_module_open(mmgui_libpaths_cache_get_library_full_path(libcache, "libcanberra"), G_MODULE_BIND_LAZY);
 	
 	if (notifications->canberramodule != NULL) {
 		libopened = TRUE;
