@@ -1,7 +1,7 @@
 /*
  *      sms-page.c
  *      
- *      Copyright 2012-2014 Alex <alex@linuxonly.ru>
+ *      Copyright 2012-2015 Alex <alex@linuxonly.ru>
  *      
  *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -33,14 +33,17 @@
 #include "strformat.h"
 #include "mmguicore.h"
 #include "smsdb.h"
-
 #include "contacts-page.h"
-
 #include "resources.h"
-
+#include "settings.h"
 #include "sms-page.h"
 #include "main.h"
 
+#if RESOURCE_SPELLCHECKER_ENABLED
+	#include <gtkspell/gtkspell.h>
+#endif
+
+
 enum _mmgui_main_smslist_columns {
 	MMGUI_MAIN_SMSLIST_ICON = 0,
 	MMGUI_MAIN_SMSLIST_SMS,
@@ -58,8 +61,8 @@ enum _mmgui_main_new_sms_validation {
 
 enum _mmgui_main_new_sms_dialog_result {
 	MMGUI_MAIN_NEW_SMS_DIALOG_CLOSE = 0,
-	MMGUI_MAIN_NEW_SMS_DIALOG_SAVE,
-	MMGUI_MAIN_NEW_SMS_DIALOG_SEND
+	MMGUI_MAIN_NEW_SMS_DIALOG_SEND,
+	MMGUI_MAIN_NEW_SMS_DIALOG_SAVE
 };
 
 enum _mmgui_main_sms_source {
@@ -97,7 +100,7 @@ static void mmgui_main_sms_select_entry_from_list(mmgui_application_t mmguiapp,
 static void mmgui_main_sms_get_message_list_hash_destroy_notify(gpointer data);
 static void mmgui_main_sms_new_dialog_number_changed_signal(GtkEditable *editable, gpointer data);
 static enum _mmgui_main_new_sms_dialog_result mmgui_main_sms_new_dialog(mmgui_application_t mmguiapp, const gchar *number, const gchar *text);
-static void mmgui_main_sms_list_cursor_changed_signal(GtkTreeView *tree_view, gpointer data);
+static void mmgui_main_sms_list_selection_changed_signal(GtkTreeSelection *selection, gpointer data);
 static void mmgui_main_sms_list_row_activated_signal(GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *col, gpointer data);
 static void mmgui_main_sms_add_to_list(mmgui_application_t mmguiapp, mmgui_sms_message_t sms, GtkTreeModel *model);
 static gboolean mmgui_main_sms_autocompletion_select_entry_signal(GtkEntryCompletion *widget, GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data);
@@ -185,6 +188,9 @@ static void mmgui_main_sms_select_entry_from_list(mmgui_application_t mmguiapp,
 		foldervalid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &folderiter);
 	}
 	
+	/*Unselect all rows as multiple selection mode enabled*/
+	gtk_tree_selection_unselect_all(selection);
+	
 	if (entryiter != NULL) {
 		/*Message found - select it*/
 		gtk_tree_selection_select_iter(selection, entryiter);
@@ -480,8 +486,8 @@ static void mmgui_main_sms_new_dialog_number_changed_signal(GtkEditable *editabl
 		} else if (bufferchars == 0) {
 			gtk_entry_set_icon_tooltip_markup(GTK_ENTRY(appdata->mmguiapp->window->smsnumberentry), GTK_ENTRY_ICON_SECONDARY, _("<b>SMS text is not valid</b>\n<small>Please write some text to send</small>"));
 		}
-		gtk_widget_set_sensitive(appdata->mmguiapp->window->sendsmsbutton, FALSE);
-		gtk_widget_set_sensitive(appdata->mmguiapp->window->savesmsbutton, FALSE);
+		gtk_widget_set_sensitive(appdata->mmguiapp->window->newsmssendtb, FALSE);
+		gtk_widget_set_sensitive(appdata->mmguiapp->window->newsmssavetb, FALSE);
 		*smsvalidflags = newsmsvalidflags;
 	} else if ((newnumvalid) && (bufferchars > 0)) {
 		#if GTK_CHECK_VERSION(3,10,0)
@@ -490,8 +496,8 @@ static void mmgui_main_sms_new_dialog_number_changed_signal(GtkEditable *editabl
 			gtk_entry_set_icon_from_stock(GTK_ENTRY(appdata->mmguiapp->window->smsnumberentry), GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_CAPS_LOCK_WARNING);
 		#endif
 		gtk_entry_set_icon_tooltip_markup(GTK_ENTRY(appdata->mmguiapp->window->smsnumberentry), GTK_ENTRY_ICON_SECONDARY, NULL);
-		gtk_widget_set_sensitive(appdata->mmguiapp->window->sendsmsbutton, TRUE);
-		gtk_widget_set_sensitive(appdata->mmguiapp->window->savesmsbutton, TRUE);
+		gtk_widget_set_sensitive(appdata->mmguiapp->window->newsmssendtb, TRUE);
+		gtk_widget_set_sensitive(appdata->mmguiapp->window->newsmssavetb, TRUE);
 		*smsvalidflags = newsmsvalidflags;
 	}
 }
@@ -555,6 +561,28 @@ static enum _mmgui_main_new_sms_dialog_result mmgui_main_sms_new_dialog(mmgui_ap
 	return result;
 }
 
+void mmgui_main_sms_new_send_signal(GtkToolButton *toolbutton, gpointer data)
+{
+	mmgui_application_t mmguiapp;
+	
+	mmguiapp = (mmgui_application_t)data;
+	
+	if (mmguiapp == NULL) return;
+	
+	gtk_dialog_response(GTK_DIALOG(mmguiapp->window->newsmsdialog), MMGUI_MAIN_NEW_SMS_DIALOG_SEND);
+}
+               
+void mmgui_main_sms_new_save_signal(GtkToolButton *toolbutton, gpointer data)
+{
+	mmgui_application_t mmguiapp;
+	
+	mmguiapp = (mmgui_application_t)data;
+	
+	if (mmguiapp == NULL) return;
+	
+	gtk_dialog_response(GTK_DIALOG(mmguiapp->window->newsmsdialog), MMGUI_MAIN_NEW_SMS_DIALOG_SAVE);
+}
+
 gboolean mmgui_main_sms_send(mmgui_application_t mmguiapp, const gchar *number, const gchar *text)
 {
 	GtkTextBuffer *buffer;
@@ -562,6 +590,7 @@ gboolean mmgui_main_sms_send(mmgui_application_t mmguiapp, const gchar *number,
 	gchar *resnumber, *restext;
 	enum _mmgui_main_new_sms_dialog_result result;
 	mmgui_sms_message_t message;
+	gchar *statusmsg;
 	
 	if ((mmguiapp == NULL) || (number == NULL) || (text == NULL)) return FALSE;
 	
@@ -569,57 +598,62 @@ gboolean mmgui_main_sms_send(mmgui_application_t mmguiapp, const gchar *number,
 	
 	if (buffer == NULL) return FALSE;
 	
-	//Open dialog for user interaction
+	/*Open dialog for user interaction*/
 	result = mmgui_main_sms_new_dialog(mmguiapp, number, text);
 	
 	if ((result == MMGUI_MAIN_NEW_SMS_DIALOG_SEND) || (result == MMGUI_MAIN_NEW_SMS_DIALOG_SAVE)) {
-		//Get final message number and text
+		/*Get final message number and text*/
 		resnumber = (gchar *)gtk_entry_get_text(GTK_ENTRY(mmguiapp->window->smsnumberentry));
 		gtk_text_buffer_get_bounds(buffer, &start, &end);
 		restext = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
-		//Send message
+		/*Send message*/
 		if (result == MMGUI_MAIN_NEW_SMS_DIALOG_SEND) {
+			/*Start progress menubar*/
+			statusmsg = g_strdup_printf(_("Sending SMS to number \"%s\"..."), resnumber);
+			mmgui_ui_infobar_show(mmguiapp, statusmsg, MMGUI_MAIN_INFOBAR_TYPE_PROGRESS);
+			g_free(statusmsg);
+			/*Send message*/
 			if (mmguicore_sms_send(mmguiapp->core, resnumber, restext, mmguiapp->options->smsvalidityperiod, mmguiapp->options->smsdeliveryreport)) {
-				//Form message
+				/*Form message*/
 				message = mmgui_smsdb_message_create();
 				mmgui_smsdb_message_set_number(message, resnumber);
 				mmgui_smsdb_message_set_text(message, restext, FALSE);
 				mmgui_smsdb_message_set_read(message, TRUE);
 				mmgui_smsdb_message_set_folder(message, MMGUI_SMSDB_SMS_FOLDER_SENT);
-				//Add message to database
+				/*Add message to database*/
 				mmgui_smsdb_add_sms(mmguicore_devices_get_sms_db(mmguiapp->core), message);
-				//Add message to list
+				/*Add message to list*/
 				mmgui_main_sms_add_to_list(mmguiapp, message, NULL);
-				//Free message
+				/*Free message*/
 				mmgui_smsdb_message_free(message);
 				/*Save last number*/
 				mmgui_main_sms_add_number_to_history(mmguiapp, resnumber);
-				//Start progress dialog
-				mmgui_main_ui_progress_dialog_open(mmguiapp);
-				
 				return TRUE;
 			} else {
-				mmgui_main_ui_error_dialog_open(mmguiapp, _("<b>Error sending message</b>"), _("Wrong number or device not ready"));
-				
+				/*Show menubar with error message*/
+				mmgui_ui_infobar_show_result(mmguiapp, MMGUI_MAIN_INFOBAR_RESULT_FAIL, _("Wrong number or device not ready"));
 				return FALSE;
 			}
-		//Save message
+		/*Save message*/
 		} else if (result == MMGUI_MAIN_NEW_SMS_DIALOG_SAVE) {
-			//Form message
+			/*Start progress menubar*/
+			mmgui_ui_infobar_show(mmguiapp, _("Saving SMS..."), MMGUI_MAIN_INFOBAR_TYPE_PROGRESS);
+			/*Form message*/
 			message = mmgui_smsdb_message_create();
 			mmgui_smsdb_message_set_number(message, resnumber);
 			mmgui_smsdb_message_set_text(message, restext, FALSE);
 			mmgui_smsdb_message_set_read(message, TRUE);
 			mmgui_smsdb_message_set_folder(message, MMGUI_SMSDB_SMS_FOLDER_DRAFTS);
-			//Add message to database
+			/*Add message to database*/
 			mmgui_smsdb_add_sms(mmguicore_devices_get_sms_db(mmguiapp->core), message);
-			//Add message to list
+			/*Add message to list*/
 			mmgui_main_sms_add_to_list(mmguiapp, message, NULL);
-			//Free message
+			/*Free message*/
 			mmgui_smsdb_message_free(message);
-			//Save last number
+			/*Save last number*/
 			mmgui_main_sms_add_number_to_history(mmguiapp, resnumber);
-			
+			/*Show result*/
+			mmgui_ui_infobar_show_result(mmguiapp, MMGUI_MAIN_INFOBAR_RESULT_SUCCESS, NULL);
 			return TRUE;
 		}
 	}
@@ -631,33 +665,81 @@ void mmgui_main_sms_remove(mmgui_application_t mmguiapp)
 {
 	GtkTreeModel *model;
 	GtkTreeSelection *selection;
+	GList *selected;
+	GList *listiter;
+	gchar *pathstr;
 	GtkTreeIter iter;
+	GtkTreeRowReference *rowref;
+	GtkTreePath *treepath;
+	GList *rowreflist;
+	guint rowcount;
+	guint rmcount;
+	gchar *questionstr;
+	gchar *errorstr;
 	gulong id;
-	/*gboolean valid;*/
 	gboolean isfolder;
 	
 	if (mmguiapp == NULL) return;
-	if (mmguicore_devices_get_current(mmguiapp->core) == NULL) return;
 		
 	model = gtk_tree_view_get_model(GTK_TREE_VIEW(mmguiapp->window->smslist));
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(mmguiapp->window->smslist));
 	
 	if ((model != NULL) && (selection != NULL)) {
-		if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
-			gtk_tree_model_get(model, &iter, MMGUI_MAIN_SMSLIST_ID, &id, MMGUI_MAIN_SMSLIST_ISFOLDER, &isfolder, -1);
-			if (!isfolder) {
-				if (mmgui_main_ui_question_dialog_open(mmguiapp, _("<b>Remove message</b>"), _("Really want to remove message?"))) {
-					if (mmgui_smsdb_remove_sms_message(mmguicore_devices_get_sms_db(mmguiapp->core), id)) {
-						gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
-						/*Ayatana menu*/
-						mmgui_ayatana_set_unread_messages_number(mmguiapp->ayatana, mmgui_smsdb_get_unread_messages(mmguicore_devices_get_sms_db(mmguiapp->core)));
-					} else {
-						mmgui_main_ui_error_dialog_open(mmguiapp, _("<b>Error removing SMS</b>"), _("Message not found"));
+		selected = gtk_tree_selection_get_selected_rows(selection, &model);
+		if (selected != NULL) {
+			/*Get selected iter and find messages*/
+			rowreflist = NULL;
+			rowcount = 0;
+			selected = g_list_reverse(selected);
+			for (listiter = selected; listiter; listiter = listiter->next) {
+				treepath = listiter->data;
+				pathstr = gtk_tree_path_to_string(treepath);
+				if (pathstr != NULL) {
+					if (gtk_tree_model_get_iter_from_string(model, &iter, (const gchar *)pathstr)) {
+						gtk_tree_model_get(model, &iter, MMGUI_MAIN_SMSLIST_ISFOLDER, &isfolder, -1);
+						if (!isfolder) {
+							rowref = gtk_tree_row_reference_new(model, treepath);
+							rowreflist = g_list_prepend(rowreflist, rowref);
+							rowcount++;
+						}
 					}
+					g_free(pathstr);
 				}
 			}
-		} else {
-			mmgui_main_ui_error_dialog_open(mmguiapp, _("<b>Error removing SMS</b>"), _("Message not selected"));
+			/*Work with collected row references*/
+			if ((rowcount > 0) && (rowreflist != NULL)) {
+				questionstr = g_strdup_printf(_("Really want to remove messages (%u) ?"), rowcount);
+				if (mmgui_main_ui_question_dialog_open(mmguiapp, _("<b>Remove messages</b>"), questionstr)) {
+					rmcount = 0;
+					for (listiter = rowreflist; listiter; listiter = listiter->next) {
+						treepath = gtk_tree_row_reference_get_path((GtkTreeRowReference *)listiter->data);
+						if (treepath != NULL) {
+							if (gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter, treepath)) {
+								gtk_tree_model_get(model, &iter, MMGUI_MAIN_SMSLIST_ID, &id, MMGUI_MAIN_SMSLIST_ISFOLDER, &isfolder, -1);
+								if (!isfolder) {
+									if (mmgui_smsdb_remove_sms_message(mmguicore_devices_get_sms_db(mmguiapp->core), id)) {
+										/*Remove entry from list*/
+										gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
+										/*Ayatana menu*/
+										mmgui_ayatana_set_unread_messages_number(mmguiapp->ayatana, mmgui_smsdb_get_unread_messages(mmguicore_devices_get_sms_db(mmguiapp->core)));
+										/*Count message*/
+										rmcount++;
+									}
+								}
+							}
+						}
+					}
+					/*Show error message if some messages weren't removed*/
+					if (rmcount < rowcount) {
+						errorstr = g_strdup_printf(_("Some messages weren\'t removed (%u)"), rowcount - rmcount);
+						mmgui_ui_infobar_show_result(mmguiapp, MMGUI_MAIN_INFOBAR_RESULT_FAIL, errorstr);
+						g_free(errorstr);						
+					}
+				}
+				g_list_foreach(rowreflist, (GFunc)gtk_tree_row_reference_free, NULL);
+				g_list_free(rowreflist);
+				g_free(questionstr);
+			}
 		}
 	}
 }
@@ -706,6 +788,10 @@ void mmgui_main_sms_answer(mmgui_application_t mmguiapp)
 {
 	GtkTreeModel *model;
 	GtkTreeSelection *selection;
+	GList *selected;
+	GList *listiter;
+	GtkTreePath *treepath;
+	gchar *pathstr;
 	GtkTreeIter iter;
 	guint smscaps;
 	gulong id;
@@ -714,29 +800,42 @@ void mmgui_main_sms_answer(mmgui_application_t mmguiapp)
 	mmgui_sms_message_t message;
 	
 	if (mmguiapp == NULL) return;
-	if (mmguicore_devices_get_current(mmguiapp->core) == NULL) return;
 		
 	model = gtk_tree_view_get_model(GTK_TREE_VIEW(mmguiapp->window->smslist));
+	
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(mmguiapp->window->smslist));
+	
 	smscaps = mmguicore_sms_get_capabilities(mmguiapp->core);
 	
-	if ((model != NULL) && (selection != NULL)) {
-		//Get selected message
-		if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
-			gtk_tree_model_get(model, &iter, MMGUI_MAIN_SMSLIST_ID, &id, MMGUI_MAIN_SMSLIST_FOLDER, &folder, MMGUI_MAIN_SMSLIST_ISFOLDER, &isfolder, -1);
-			if (!isfolder) {
-				message = mmgui_smsdb_read_sms_message(mmguicore_devices_get_sms_db(mmguiapp->core), id);
-				if (message != NULL) {
-					//Open dialog
-					if (mmguicore_sms_validate_number(mmgui_smsdb_message_get_number(message)) && (smscaps & MMGUI_SMS_CAPS_SEND)) {
-						if (folder == MMGUI_SMSDB_SMS_FOLDER_DRAFTS) {
-							mmgui_main_sms_send(mmguiapp, mmgui_smsdb_message_get_number(message), mmgui_smsdb_message_get_text(message));
-						} else {
-							mmgui_main_sms_send(mmguiapp, mmgui_smsdb_message_get_number(message), "");
+	if ((model != NULL) && (selection != NULL) && (smscaps & MMGUI_SMS_CAPS_SEND)) {
+		/*Get selected messages and select first one*/
+		selected = gtk_tree_selection_get_selected_rows(selection, &model);
+		if (selected != NULL) {
+			selected = g_list_reverse(selected);
+			for (listiter = selected; listiter; listiter = listiter->next) {
+				treepath = listiter->data;
+				pathstr = gtk_tree_path_to_string(treepath);
+				if (pathstr != NULL) {
+					if (gtk_tree_model_get_iter_from_string(model, &iter, (const gchar *)pathstr)) {
+						gtk_tree_model_get(model, &iter, MMGUI_MAIN_SMSLIST_ID, &id, MMGUI_MAIN_SMSLIST_FOLDER, &folder, MMGUI_MAIN_SMSLIST_ISFOLDER, &isfolder, -1);
+						if (!isfolder) {
+							message = mmgui_smsdb_read_sms_message(mmguicore_devices_get_sms_db(mmguiapp->core), id);
+							if (message != NULL) {
+								/*Open dialog*/
+								if (mmguicore_sms_validate_number(mmgui_smsdb_message_get_number(message))) {
+									if (folder == MMGUI_SMSDB_SMS_FOLDER_DRAFTS) {
+										mmgui_main_sms_send(mmguiapp, mmgui_smsdb_message_get_number(message), mmgui_smsdb_message_get_text(message));
+									} else {
+										mmgui_main_sms_send(mmguiapp, mmgui_smsdb_message_get_number(message), "");
+									}
+									break;
+								}
+								/*Free message*/
+								mmgui_smsdb_message_free(message);
+							}
 						}
 					}
-					//Free message
-					mmgui_smsdb_message_free(message);
+					g_free(pathstr);
 				}
 			}
 		}
@@ -754,86 +853,215 @@ void mmgui_main_sms_answer_button_clicked_signal(GObject *object, gpointer data)
 	mmgui_main_sms_answer(mmguiapp);
 }
 
-static void mmgui_main_sms_list_cursor_changed_signal(GtkTreeView *tree_view, gpointer data)
+static void mmgui_main_sms_list_selection_changed_signal(GtkTreeSelection *selection, gpointer data)
 {
 	mmgui_application_t mmguiapp;
+	mmgui_sms_message_t sms;
 	guint smscaps;
 	GtkTreeModel *model;
-	GtkTreeSelection *selection;
-	GtkTextBuffer *buffer;
+	GList *selected;
+	GList *listiter;
+	gchar *pathstr;
+	GtkTextIter starttextiter, endtextiter;
+	GtkTextBuffer *textbuffer;
+	time_t timestamp;
+	gchar timestr[200];
 	GtkTreeIter iter;
-	/*GtkTextIter siter, eiter;*/
-	gulong id;
-	guint folder;
-	gboolean isfolder;
-	mmgui_sms_message_t sms;
+	GtkTreeRowReference *rowref;
+	GtkTreePath *treepath;
+	GList *msgreflist, *folderreflist;
+	guint msgcount, foldercount, folder;
+	gulong id, lastid;
+	gboolean isfolder, numchecked, canremove, cananswer;
+	gchar *msgnumber;
 	
 	mmguiapp = (mmgui_application_t)data;
 	
 	if (mmguiapp == NULL) return;
 	
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(mmguiapp->window->smslist));
+	
+	textbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(mmguiapp->window->smstext));
+	
 	smscaps = mmguicore_sms_get_capabilities(mmguiapp->core);
 	
-	model = gtk_tree_view_get_model(tree_view);
-	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
-	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(mmguiapp->window->smstext));
+	if (textbuffer != NULL) {
+		/*Clear text*/
+		gtk_text_buffer_get_bounds(textbuffer, &starttextiter, &endtextiter);
+		gtk_text_buffer_delete(textbuffer, &starttextiter, &endtextiter);
+	}
 	
-	if (mmguicore_devices_get_current(mmguiapp->core) != NULL) {
-		if ((model != NULL) && (buffer != NULL)) {
-			if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
-				gtk_tree_model_get(model, &iter, MMGUI_MAIN_SMSLIST_ID, &id, MMGUI_MAIN_SMSLIST_FOLDER, &folder, MMGUI_MAIN_SMSLIST_ISFOLDER, &isfolder, -1);
-				if (!isfolder) {
-					sms = mmgui_smsdb_read_sms_message(mmguicore_devices_get_sms_db(mmguiapp->core), id);
-					if (sms != NULL) {
-						//Show message text
-						gtk_text_buffer_set_text(buffer, mmgui_smsdb_message_get_text(sms), -1);
-						//Enable Answer button if needed
-						if (mmguicore_sms_validate_number(mmgui_smsdb_message_get_number(sms)) && (smscaps & MMGUI_SMS_CAPS_SEND)) {
-							gtk_widget_set_sensitive(mmguiapp->window->answersmsbutton, TRUE);
+	if ((model != NULL) && (selection != NULL) && (textbuffer != NULL)) {
+		selected = gtk_tree_selection_get_selected_rows(selection, &model);
+		if (selected != NULL) {
+			/*Get selected iter and find messages and folders*/
+			msgreflist = NULL;
+			msgcount = 0;
+			folderreflist = NULL;
+			foldercount = 0;
+			canremove = FALSE;
+			cananswer = FALSE;
+			selected = g_list_reverse(selected);
+			for (listiter = selected; listiter; listiter = listiter->next) {
+				treepath = listiter->data;
+				pathstr = gtk_tree_path_to_string(treepath);
+				if (pathstr != NULL) {
+					if (gtk_tree_model_get_iter_from_string(model, &iter, (const gchar *)pathstr)) {
+						gtk_tree_model_get(model, &iter, MMGUI_MAIN_SMSLIST_ISFOLDER, &isfolder, -1);
+						rowref = gtk_tree_row_reference_new(model, treepath);
+						if (isfolder) {
+							folderreflist = g_list_prepend(folderreflist, rowref);
+							foldercount++;
 						} else {
-							gtk_widget_set_sensitive(mmguiapp->window->answersmsbutton, FALSE);
+							msgreflist = g_list_prepend(msgreflist, rowref);
+							msgcount++;							
 						}
-						gtk_widget_set_sensitive(mmguiapp->window->removesmsbutton, TRUE);
-						//Set read flag if not set before
-						if (!mmgui_smsdb_message_get_read(sms)) {
-							if (mmgui_smsdb_set_message_read_status(mmguicore_devices_get_sms_db(mmguiapp->core), id, TRUE)) {
-								gtk_tree_store_set(GTK_TREE_STORE(model), &iter, MMGUI_MAIN_SMSLIST_ICON, mmguiapp->window->smsreadicon, -1);
+					}
+					g_free(pathstr);
+				}
+			}
+			
+			lastid = 0;
+			
+			/*Show messages using collected row references*/
+			if ((msgcount > 0) && (msgreflist != NULL)) {
+				numchecked = FALSE;
+				msgnumber = NULL;
+				for (listiter = msgreflist; listiter; listiter = listiter->next) {
+					treepath = gtk_tree_row_reference_get_path((GtkTreeRowReference *)listiter->data);
+					if (treepath != NULL) {
+						if (gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter, treepath)) {
+							gtk_tree_model_get(model, &iter, MMGUI_MAIN_SMSLIST_ID, &id, MMGUI_MAIN_SMSLIST_ISFOLDER, &isfolder, -1);
+							if (!isfolder) {
+								sms = mmgui_smsdb_read_sms_message(mmguicore_devices_get_sms_db(mmguiapp->core), id);
+								if (sms != NULL) {
+									/*Message heading*/
+									gtk_text_buffer_get_end_iter(textbuffer, &endtextiter);
+									gtk_text_buffer_insert_with_tags(textbuffer, &endtextiter, mmgui_smsdb_message_get_number(sms), -1, mmguiapp->window->smsheadingtag, NULL);
+									/*Timestamp*/
+									timestamp = mmgui_smsdb_message_get_timestamp(sms);
+									gtk_text_buffer_get_end_iter(textbuffer, &endtextiter);
+									gtk_text_buffer_insert_with_tags(textbuffer, &endtextiter, mmgui_str_format_sms_time(timestamp, timestr, sizeof(timestr)), -1, mmguiapp->window->smsdatetag, NULL);
+									gtk_text_buffer_get_end_iter(textbuffer, &endtextiter);
+									gtk_text_buffer_insert(textbuffer, &endtextiter, "\n", -1);
+									/*Message text*/
+									gtk_text_buffer_get_end_iter(textbuffer, &endtextiter);
+									gtk_text_buffer_insert(textbuffer, &endtextiter, mmgui_smsdb_message_get_text(sms), -1);
+									gtk_text_buffer_get_end_iter(textbuffer, &endtextiter);
+									gtk_text_buffer_insert(textbuffer, &endtextiter, "\n\n", -1);
+									
+									/*Test if sender number is available*/
+									if (smscaps & MMGUI_SMS_CAPS_SEND) {
+										if (!numchecked) {
+											if (mmguicore_sms_validate_number(mmgui_smsdb_message_get_number(sms)) && (smscaps & MMGUI_SMS_CAPS_SEND)) {
+												msgnumber = g_strdup(mmgui_smsdb_message_get_number(sms));
+												cananswer = TRUE;
+											}
+											numchecked = TRUE;
+										} else {
+											if (cananswer) {
+												if (mmgui_smsdb_message_get_number(sms) != NULL) {
+													if (!g_str_equal(msgnumber, mmgui_smsdb_message_get_number(sms))) {
+														cananswer = FALSE;
+													}
+												} else {
+													cananswer = FALSE;
+												}
+											}
+										}
+									}
+									
+									canremove = TRUE;
+									
+									/*Set read flag if not set before*/
+									if (!mmgui_smsdb_message_get_read(sms)) {
+										if (mmgui_smsdb_set_message_read_status(mmguicore_devices_get_sms_db(mmguiapp->core), id, TRUE)) {
+											gtk_tree_store_set(GTK_TREE_STORE(model), &iter, MMGUI_MAIN_SMSLIST_ICON, mmguiapp->window->smsreadicon, -1);
+										}
+										/*Ayatana menu*/
+										mmgui_ayatana_set_unread_messages_number(mmguiapp->ayatana, mmgui_smsdb_get_unread_messages(mmguicore_devices_get_sms_db(mmguiapp->core)));
+									}
+									/*Save last message ID*/
+									lastid = id;
+									/*Free SMS message*/
+									mmgui_smsdb_message_free(sms);
+								}
 							}
-							/*Ayatana menu*/
-							mmgui_ayatana_set_unread_messages_number(mmguiapp->ayatana, mmgui_smsdb_get_unread_messages(mmguicore_devices_get_sms_db(mmguiapp->core)));
 						}
-						//Free SMS message
-						mmgui_smsdb_message_free(sms);
-						/*Save identifier*/
-						mmgui_modem_settings_set_boolean(mmguiapp->modemsettings, "sms_is_folder", FALSE);
-						mmgui_modem_settings_set_int64(mmguiapp->modemsettings, "sms_entry_id", (gint64)id);
-					
-					} else {
-						//Cant read message
-						gtk_widget_set_sensitive(mmguiapp->window->answersmsbutton, FALSE);
-						gtk_text_buffer_set_text(buffer, _("This message can't be read"), -1);
 					}
-				} else {
-					//Folder selected
-					switch (folder) {
-						case MMGUI_SMSDB_SMS_FOLDER_INCOMING:
-							gtk_text_buffer_set_text(buffer, _("This is folder for your incoming SMS messages.\nYou can answer selected message using 'Answer' button."), -1);
-							break;
-						case MMGUI_SMSDB_SMS_FOLDER_SENT:
-							gtk_text_buffer_set_text(buffer, _("This is folder for your sent SMS messages."), -1);
-							break;
-						case MMGUI_SMSDB_SMS_FOLDER_DRAFTS:
-							gtk_text_buffer_set_text(buffer, _("This is folder for your SMS message drafts.\nSelect message and click 'Answer' button to start editing."), -1);
-							break;
-						default:
-							break;
+				}
+				g_list_foreach(msgreflist, (GFunc)gtk_tree_row_reference_free, NULL);
+				g_list_free(msgreflist);
+				if (msgnumber != NULL) {
+					g_free(msgnumber);
+				}
+				/*Save selected message identifier*/
+				if (lastid != 0) {
+					mmgui_modem_settings_set_boolean(mmguiapp->modemsettings, "sms_is_folder", FALSE);
+					mmgui_modem_settings_set_int64(mmguiapp->modemsettings, "sms_entry_id", lastid);
+				}
+			}
+			
+			/*Enable Answer and remove buttons if needed*/
+			gtk_widget_set_sensitive(mmguiapp->window->answersmsbutton, cananswer);
+			gtk_widget_set_sensitive(mmguiapp->window->removesmsbutton, canremove);
+			
+			/*Show folders descriptions using collected row references*/
+			if ((foldercount > 0) && (folderreflist != NULL)) {
+				/*Show tips only if no messages selected*/
+				if (lastid == 0) {
+					for (listiter = folderreflist; listiter; listiter = listiter->next) {
+						treepath = gtk_tree_row_reference_get_path((GtkTreeRowReference *)listiter->data);
+						if (treepath != NULL) {
+							if (gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter, treepath)) {
+								gtk_tree_model_get(model, &iter, MMGUI_MAIN_SMSLIST_FOLDER, &folder, MMGUI_MAIN_SMSLIST_ISFOLDER, &isfolder, -1);
+								if (isfolder) {
+									/*Folder selected*/
+									switch (folder) {
+										case MMGUI_SMSDB_SMS_FOLDER_INCOMING:
+											/*Folder heading*/
+											gtk_text_buffer_get_end_iter(textbuffer, &endtextiter);
+											gtk_text_buffer_insert_with_tags(textbuffer, &endtextiter, _("Incoming"), -1, mmguiapp->window->smsheadingtag, NULL);
+											gtk_text_buffer_get_end_iter(textbuffer, &endtextiter);
+											gtk_text_buffer_insert(textbuffer, &endtextiter, "\n", -1);
+											/*Folder description*/
+											gtk_text_buffer_get_end_iter(textbuffer, &endtextiter);
+											gtk_text_buffer_insert(textbuffer, &endtextiter, _("This is folder for your incoming SMS messages.\nYou can answer selected message using 'Answer' button."), -1);
+											break;
+										case MMGUI_SMSDB_SMS_FOLDER_SENT:
+											/*Folder heading*/
+											gtk_text_buffer_get_end_iter(textbuffer, &endtextiter);
+											gtk_text_buffer_insert_with_tags(textbuffer, &endtextiter, _("Sent"), -1, mmguiapp->window->smsheadingtag, NULL);
+											gtk_text_buffer_get_end_iter(textbuffer, &endtextiter);
+											gtk_text_buffer_insert(textbuffer, &endtextiter, "\n", -1);
+											/*Folder description*/
+											gtk_text_buffer_get_end_iter(textbuffer, &endtextiter);
+											gtk_text_buffer_insert(textbuffer, &endtextiter, _("This is folder for your sent SMS messages."), -1);
+											break;
+										case MMGUI_SMSDB_SMS_FOLDER_DRAFTS:
+											/*Folder heading*/
+											gtk_text_buffer_get_end_iter(textbuffer, &endtextiter);
+											gtk_text_buffer_insert_with_tags(textbuffer, &endtextiter, _("Drafts"), -1, mmguiapp->window->smsheadingtag, NULL);
+											gtk_text_buffer_get_end_iter(textbuffer, &endtextiter);
+											gtk_text_buffer_insert(textbuffer, &endtextiter, "\n", -1);
+											/*Folder description*/
+											gtk_text_buffer_get_end_iter(textbuffer, &endtextiter);
+											gtk_text_buffer_insert(textbuffer, &endtextiter, _("This is folder for your SMS message drafts.\nSelect message and click 'Answer' button to start editing."), -1);
+											break;
+										default:
+											break;
+									}
+								}
+							}
+						}
 					}
-					gtk_widget_set_sensitive(mmguiapp->window->removesmsbutton, FALSE);
-					gtk_widget_set_sensitive(mmguiapp->window->answersmsbutton, FALSE);
-					/*Save folder*/
-					mmgui_modem_settings_set_boolean(mmguiapp->modemsettings, "sms_is_folder", TRUE);
-					mmgui_modem_settings_set_int64(mmguiapp->modemsettings, "sms_entry_id", (gint64)folder);
-					
+				}
+				g_list_foreach(folderreflist, (GFunc)gtk_tree_row_reference_free, NULL);
+				g_list_free(folderreflist);
+				/*Save selected folder identifier*/
+				if (lastid == 0) {
+					mmgui_modem_settings_set_boolean(mmguiapp->modemsettings, "sms_is_folder", FALSE);
+					mmgui_modem_settings_set_int64(mmguiapp->modemsettings, "sms_entry_id", id);
 				}
 			}
 		}
@@ -1438,11 +1666,168 @@ static gchar *mmgui_main_sms_list_select_entry_signal(GtkComboBox *combo, const
 	return number;
 }
 
+#if RESOURCE_SPELLCHECKER_ENABLED
+void mmgui_main_sms_new_disable_spellchecker_signal(GtkToolButton *toolbutton, gpointer data)
+{
+	mmgui_application_t mmguiapp;
+	const gchar *langcode;
+	gchar *langname;
+	
+	mmguiapp = (mmgui_application_t)data;
+	
+	if (mmguiapp == NULL) return;
+	if (mmguiapp->window->newsmsspellchecker == NULL) return;
+	
+	if (gtk_spell_checker_get_from_text_view(GTK_TEXT_VIEW(mmguiapp->window->smstextview)) != NULL) {
+		/*Detach spellchecker*/
+		g_object_ref(mmguiapp->window->newsmsspellchecker);
+		gtk_spell_checker_detach(mmguiapp->window->newsmsspellchecker);
+		gmm_settings_set_boolean(mmguiapp->settings, "spell_checker_enabled", FALSE);
+		gtk_tool_button_set_label(GTK_TOOL_BUTTON(mmguiapp->window->newsmsspellchecktb), _("Disabled"));
+	} else {
+		/*Attach spellchecker*/
+		gtk_spell_checker_attach(mmguiapp->window->newsmsspellchecker, GTK_TEXT_VIEW(mmguiapp->window->smstextview));
+		g_object_unref(mmguiapp->window->newsmsspellchecker);
+		gmm_settings_set_boolean(mmguiapp->settings, "spell_checker_enabled", TRUE);
+		langcode = gtk_spell_checker_get_language(mmguiapp->window->newsmsspellchecker);
+		langname = gtk_spell_checker_decode_language_code(langcode);
+		gtk_tool_button_set_label(GTK_TOOL_BUTTON(mmguiapp->window->newsmsspellchecktb), langname);
+		g_free(langname);
+	}
+}
+
+static void mmgui_main_sms_spellcheck_menu_activate_signal(GtkMenuItem *menuitem, gpointer data)
+{
+	mmgui_application_data_t appdata;
+	gchar *langname;
+		
+	appdata = (mmgui_application_data_t)data;
+	
+	if (appdata == NULL) return;
+	
+	if (gtk_spell_checker_get_from_text_view(GTK_TEXT_VIEW(appdata->mmguiapp->window->smstextview)) == NULL) {
+		/*First attach spell checker if disabled*/
+		gtk_spell_checker_attach(appdata->mmguiapp->window->newsmsspellchecker, GTK_TEXT_VIEW(appdata->mmguiapp->window->smstextview));
+		g_object_unref(appdata->mmguiapp->window->newsmsspellchecker);
+		gmm_settings_set_boolean(appdata->mmguiapp->settings, "spell_checker_enabled", TRUE);
+	}
+	
+	/*Then set language*/
+	langname = gtk_spell_checker_decode_language_code((const gchar *)appdata->data);
+	gtk_spell_checker_set_language(appdata->mmguiapp->window->newsmsspellchecker, appdata->data, NULL);
+	gmm_settings_set_string(appdata->mmguiapp->settings, "spell_checker_language", appdata->data);
+	gtk_tool_button_set_label(GTK_TOOL_BUTTON(appdata->mmguiapp->window->newsmsspellchecktb), langname);
+	g_free(langname);
+}
+
+gboolean mmgui_main_sms_spellcheck_init(mmgui_application_t mmguiapp)
+{
+	GList *languages, *iterator;
+	mmgui_application_data_t appdata;
+	gint i, localenum;
+	gchar *locale, *locpart, *userlang, *langname, *langcode;
+	GtkWidget *langmenuentry;
+	gboolean langset;
+	
+	/*Create spell checker*/
+	mmguiapp->window->newsmsspellchecker = gtk_spell_checker_new();
+	/*Fill combo box with languages*/
+	languages = gtk_spell_checker_get_language_list();
+	if (languages != NULL) {
+		/*Create menu widget*/
+		mmguiapp->window->newsmslangmenu = gtk_menu_new();
+		/*Get current locale*/
+		locale = getenv("LANG");
+		if (locale != NULL) {
+			locale = g_strdup(locale);
+			locpart = strchr(locale, '.');
+			/*Drop encoding part if any*/
+			if (locpart != NULL) {
+				locpart[0] = '\0';
+			}
+		} else {
+			locale = g_strdup("en_US");
+		}
+		/*Get selected language*/
+		userlang = gmm_settings_get_string(mmguiapp->settings, "spell_checker_language", locale);
+		langset = FALSE;
+		localenum = -1;
+		i = 0;
+		for (iterator = languages; iterator != NULL; iterator = iterator->next) {
+			/*Get language code and name*/
+			langcode = (gchar *)iterator->data;
+			langname = gtk_spell_checker_decode_language_code((const gchar *)langcode);
+			/*Create menu entry*/
+			langmenuentry = gtk_menu_item_new_with_label(langname);
+			gtk_menu_shell_append(GTK_MENU_SHELL(mmguiapp->window->newsmslangmenu), langmenuentry);
+			/*Set signal*/
+			appdata = g_new(struct _mmgui_application_data, 1);
+			appdata->mmguiapp = mmguiapp;
+			appdata->data = g_strdup(langcode);
+			g_signal_connect(G_OBJECT(langmenuentry), "activate", G_CALLBACK(mmgui_main_sms_spellcheck_menu_activate_signal), appdata);
+			mmguiapp->window->newsmscodes = g_slist_prepend(mmguiapp->window->newsmscodes, appdata);
+			/*Set default language*/
+			if (g_str_equal(langcode, userlang)) {
+				gtk_spell_checker_set_language(mmguiapp->window->newsmsspellchecker, langcode, NULL);
+				if (gmm_settings_get_boolean(mmguiapp->settings, "spell_checker_enabled", TRUE)) {
+					gtk_tool_button_set_label(GTK_TOOL_BUTTON(mmguiapp->window->newsmsspellchecktb), langname);
+				}
+				gtk_widget_set_sensitive(mmguiapp->window->newsmsspellchecktb, TRUE);
+				langset = TRUE;
+			}
+			/*Try to get locale-specific number*/
+			if ((!langset) && (localenum == -1)) {
+				if (g_str_equal(langcode, locale)) {
+					localenum = i;
+				}
+			}
+			/*Free language name*/
+			g_free(langname);
+			i++;
+		}
+		/*Set first language or locale*/
+		if (!langset) {
+			if (localenum != -1) {
+				langcode = (gchar *)g_list_nth_data(languages, localenum);
+			} else {
+				langcode = (gchar *)g_list_nth_data(languages, 0);
+			}
+			langname = gtk_spell_checker_decode_language_code((const gchar *)langcode);
+			gtk_spell_checker_set_language(mmguiapp->window->newsmsspellchecker, langcode, NULL);
+			if (gmm_settings_get_boolean(mmguiapp->settings, "spell_checker_enabled", TRUE)) {
+				gtk_tool_button_set_label(GTK_TOOL_BUTTON(mmguiapp->window->newsmsspellchecktb), langname);
+			}
+			gtk_widget_set_sensitive(mmguiapp->window->newsmsspellchecktb, TRUE);
+			g_free(langname);
+		}
+		/*Show menu*/
+		gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(mmguiapp->window->newsmsspellchecktb), mmguiapp->window->newsmslangmenu);
+		gtk_widget_show_all(mmguiapp->window->newsmslangmenu);
+		/*Activate spell checker*/
+		gtk_spell_checker_attach(mmguiapp->window->newsmsspellchecker, GTK_TEXT_VIEW(mmguiapp->window->smstextview));
+		if (!gmm_settings_get_boolean(mmguiapp->settings, "spell_checker_enabled", TRUE)) {
+			/*Attach and detach spell checker*/
+			g_object_ref(mmguiapp->window->newsmsspellchecker);
+			gtk_spell_checker_detach(mmguiapp->window->newsmsspellchecker);
+		}
+		/*Free resources*/
+		g_list_foreach(languages, (GFunc)g_free, NULL);
+		g_list_free(languages);
+		g_free(userlang);
+		g_free(locale);
+	}
+	
+	return TRUE;
+}
+#endif
+
 void mmgui_main_sms_list_init(mmgui_application_t mmguiapp)
 {
 	GtkCellRenderer *renderer;
+	GtkTreeSelection *selection;
 	GtkTreeViewColumn *column;
 	GtkTreeStore *store;
+	GtkTextBuffer *textbuffer;
 			
 	if (mmguiapp == NULL) return;
 	
@@ -1463,8 +1848,16 @@ void mmgui_main_sms_list_init(mmgui_application_t mmguiapp)
 	gtk_tree_view_set_model(GTK_TREE_VIEW(mmguiapp->window->smslist), GTK_TREE_MODEL(store));
 	g_object_unref(store);
 	
+	/*Create textview formatting tags*/
+	textbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(mmguiapp->window->smstext));
+	
+	mmguiapp->window->smsheadingtag = gtk_text_buffer_create_tag(textbuffer, "smsheading", "weight", PANGO_WEIGHT_BOLD, "size", 15 * PANGO_SCALE, NULL);
+	mmguiapp->window->smsdatetag = gtk_text_buffer_create_tag(textbuffer, "xx-small", "scale", PANGO_SCALE_XX_SMALL, NULL);
+	
 	/*Open message signal*/
-	g_signal_connect(G_OBJECT(mmguiapp->window->smslist), "cursor-changed", G_CALLBACK(mmgui_main_sms_list_cursor_changed_signal), mmguiapp);
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(mmguiapp->window->smslist));
+	g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(mmgui_main_sms_list_selection_changed_signal), mmguiapp);
+	
 	/*Answer message signal*/
 	g_signal_connect(G_OBJECT(mmguiapp->window->smslist), "row-activated", G_CALLBACK(mmgui_main_sms_list_row_activated_signal), mmguiapp);
 	
