@@ -270,6 +270,10 @@ gchar *mmgui_str_format_access_tech_string(enum _mmgui_access_tech status)
 			return "HSUPA";
 		case MMGUI_ACCESS_TECH_HSPA:
 			return "HSPA";
+		case MMGUI_ACCESS_TECH_HSPA_PLUS:
+			return "HSPA+";
+		case MMGUI_ACCESS_TECH_LTE:
+			return "LTE";
 		default:
 			return "Unknown";
 	}
