@@ -1,7 +1,7 @@
 /*
  *      traffic-page.c
  *      
- *      Copyright 2012-2014 Alex <alex@linuxonly.ru>
+ *      Copyright 2012-2015 Alex <alex@linuxonly.ru>
  *      
  *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -651,16 +651,16 @@ void mmgui_main_traffic_limits_dialog(mmgui_application_t mmguiapp)
 			
 			switch (mmguiapp->coreoptions->trafficunits) {
 				case 0:
-					mmguiapp->coreoptions->trafficfull = mmguiapp->coreoptions->trafficamount*1024*1024;
+					mmguiapp->coreoptions->trafficfull = (guint64)mmguiapp->coreoptions->trafficamount*1024*1024;
 					break;
 				case 1:
-					mmguiapp->coreoptions->trafficfull = mmguiapp->coreoptions->trafficamount*1024*1024*1024;
+					mmguiapp->coreoptions->trafficfull = (guint64)mmguiapp->coreoptions->trafficamount*1024*1024*1024;
 					break;
 				case 2:
-					mmguiapp->coreoptions->trafficfull = mmguiapp->coreoptions->trafficamount*1024*1024*1024*1024;
+					mmguiapp->coreoptions->trafficfull = (guint64)mmguiapp->coreoptions->trafficamount*1024*1024*1024*1024;
 					break;
 				default:
-					mmguiapp->coreoptions->trafficfull = mmguiapp->coreoptions->trafficamount*1024*1024;
+					mmguiapp->coreoptions->trafficfull = (guint64)mmguiapp->coreoptions->trafficamount*1024*1024;
 					break;
 			}
 			
@@ -757,124 +757,18 @@ void mmgui_main_traffic_limits_dialog_button_clicked_signal(GObject *object, gpo
 	mmgui_main_traffic_limits_dialog(mmguiapp);
 }
 
-gboolean mmgui_main_traffic_stats_update_from_thread_foreach(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
+gboolean mmgui_main_traffic_stats_update_from_thread(gpointer data)
 {
 	mmgui_application_t mmguiapp;
+	mmgui_trafficdb_t trafficdb;
 	mmguidevice_t device;
+	GtkTreeModel *model;
+	GtkTreeIter sectioniter, elementiter;
+	gboolean sectionvalid, elementvalid;
 	gint id;
 	gchar buffer[64];
 	gfloat speed;
 	guint64 limitleft;
-		
-	mmguiapp = (mmgui_application_t)data;
-	
-	if (mmguiapp == NULL) return FALSE;
-	if (mmguiapp->core == NULL) return FALSE;
-	
-	device = mmguicore_devices_get_current(mmguiapp->core);
-	
-	if ((device == NULL) || ((device != NULL) && (!device->connected))) {
-		gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, "", -1);
-	} else {
-		gtk_tree_model_get(model, iter, MMGUI_MAIN_TRAFFICLIST_ID, &id, -1);
-		switch (id) {
-			case MMGUI_MAIN_TRAFFICLIST_ID_RXDATA:
-				gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_bytes(device->rxbytes, buffer, sizeof(buffer), TRUE), -1);
-				break;
-			case MMGUI_MAIN_TRAFFICLIST_ID_TXDATA:
-				gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_bytes(device->txbytes, buffer, sizeof(buffer), TRUE), -1);
-				break;
-			case MMGUI_MAIN_TRAFFICLIST_ID_RXSPEED:
-				if (device->speedindex < MMGUI_SPEED_VALUES_NUMBER) {
-					if (device->speedindex == 0) {
-						speed = device->speedvalues[0][device->speedindex];
-					} else {
-						speed = device->speedvalues[0][device->speedindex-1];
-					}
-				} else {
-					speed = device->speedvalues[0][MMGUI_SPEED_VALUES_NUMBER-1];
-				}
-				gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_speed(speed, buffer, sizeof(buffer), TRUE), -1);
-				break;
-			case MMGUI_MAIN_TRAFFICLIST_ID_TXSPEED:
-				if (device->speedindex < MMGUI_SPEED_VALUES_NUMBER) {
-					if (device->speedindex == 0) {
-						speed = device->speedvalues[1][device->speedindex];
-					} else {
-						speed = device->speedvalues[1][device->speedindex-1];
-					}
-				} else {
-					speed = device->speedvalues[1][MMGUI_SPEED_VALUES_NUMBER-1];
-				}
-				gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_speed(speed, buffer, sizeof(buffer), TRUE), -1);
-				break;
-			case MMGUI_MAIN_TRAFFICLIST_ID_TIME:
-				if (device->connected) {
-					gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_time(device->sessiontime, buffer, sizeof(buffer), TRUE), -1);
-				} else {
-					gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Disconnected</b></small>"), -1);
-				}
-				break;
-			case MMGUI_MAIN_TRAFFICLIST_ID_DATALIMIT:
-				if (device->connected) {
-					if (mmguiapp->coreoptions != NULL) {
-						if (!mmguiapp->coreoptions->trafficexecuted) {
-							if (mmguiapp->coreoptions->trafficenabled) {
-								limitleft = mmguiapp->coreoptions->trafficfull - (device->rxbytes + device->txbytes);
-								if (mmguiapp->coreoptions->trafficfull > (device->rxbytes + device->txbytes)) {
-									gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_bytes(limitleft, buffer, sizeof(buffer), TRUE), -1);
-								} else {
-									gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Limit</b></small>"), -1);
-								}
-							} else {
-								gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Disabled</b></small>"), -1);
-							}
-						} else {
-							gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Limit</b></small>"), -1);
-						}
-					} else {
-						gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Disabled</b></small>"), -1);
-					}
-				} else {
-					gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Disconnected</b></small>"), -1);
-				}
-				break;
-			case MMGUI_MAIN_TRAFFICLIST_ID_TIMELIMIT:
-				if (device->connected) {
-					if (mmguiapp->coreoptions != NULL) {
-						if (!mmguiapp->coreoptions->timeexecuted) {
-							if (mmguiapp->coreoptions->timeenabled) {
-								limitleft = mmguiapp->coreoptions->timefull - device->sessiontime;
-								if (mmguiapp->coreoptions->timefull > device->sessiontime) {
-									gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_time(limitleft, buffer, sizeof(buffer), TRUE), -1);
-								} else {
-									gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Limit</b></small>"), -1);
-								}
-							} else {
-								gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Disabled</b></small>"), -1);
-							}
-						} else {
-							gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Limit</b></small>"), -1);
-						}
-					} else {
-						gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Disabled</b></small>"), -1);
-					}
-				} else {
-					gtk_list_store_set(GTK_LIST_STORE(model), iter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Disconnected</b></small>"), -1);
-				}
-				break;
-			default:
-				break;
-		}
-	}
-	
-	return FALSE;
-}
-
-gboolean mmgui_main_traffic_stats_update_from_thread(gpointer data)
-{
-	mmgui_application_t mmguiapp;
-	GtkTreeModel *model;
 	GdkWindow *window;
 	gboolean visible;
 	
@@ -884,15 +778,143 @@ gboolean mmgui_main_traffic_stats_update_from_thread(gpointer data)
 	if (mmguiapp->core == NULL) return FALSE;
 	if (mmguiapp->core->device == NULL) return FALSE;
 	
-	//Update traffic statistics
+	trafficdb = (mmgui_trafficdb_t)mmguicore_devices_get_traffic_db(mmguiapp->core);
+	
+	/*Update traffic statistics*/
 	model = gtk_tree_view_get_model(GTK_TREE_VIEW(mmguiapp->window->trafficparamslist));
 	if (model != NULL) {
-		gtk_tree_model_foreach(model, mmgui_main_traffic_stats_update_from_thread_foreach, data);
+		sectionvalid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &sectioniter);
+		while (sectionvalid) {
+			/*Sections*/
+			if (gtk_tree_model_iter_has_child(model, &sectioniter)) {
+				/*Elements*/
+				if (gtk_tree_model_iter_children(model, &elementiter, &sectioniter)) {
+					do {
+						device = mmguicore_devices_get_current(mmguiapp->core);
+						
+						if ((device == NULL) || ((device != NULL) && (!device->connected))) {
+							gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, "", -1);
+						} else {
+							gtk_tree_model_get(model, &elementiter, MMGUI_MAIN_TRAFFICLIST_ID, &id, -1);
+							switch (id) {
+								case MMGUI_MAIN_TRAFFICLIST_ID_RXDATA:
+									gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_bytes(device->rxbytes, buffer, sizeof(buffer), TRUE), -1);
+									break;
+								case MMGUI_MAIN_TRAFFICLIST_ID_TXDATA:
+									gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_bytes(device->txbytes, buffer, sizeof(buffer), TRUE), -1);
+									break;
+								case MMGUI_MAIN_TRAFFICLIST_ID_RXSPEED:
+									if (device->speedindex < MMGUI_SPEED_VALUES_NUMBER) {
+										if (device->speedindex == 0) {
+											speed = device->speedvalues[0][device->speedindex];
+										} else {
+											speed = device->speedvalues[0][device->speedindex-1];
+										}
+									} else {
+										speed = device->speedvalues[0][MMGUI_SPEED_VALUES_NUMBER-1];
+									}
+									gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_speed(speed, buffer, sizeof(buffer), TRUE), -1);
+									break;
+								case MMGUI_MAIN_TRAFFICLIST_ID_TXSPEED:
+									if (device->speedindex < MMGUI_SPEED_VALUES_NUMBER) {
+										if (device->speedindex == 0) {
+											speed = device->speedvalues[1][device->speedindex];
+										} else {
+											speed = device->speedvalues[1][device->speedindex-1];
+										}
+									} else {
+										speed = device->speedvalues[1][MMGUI_SPEED_VALUES_NUMBER-1];
+									}
+									gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_speed(speed, buffer, sizeof(buffer), TRUE), -1);
+									break;
+								case MMGUI_MAIN_TRAFFICLIST_ID_TIME:
+									if (device->connected) {
+										gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_time(device->sessiontime, buffer, sizeof(buffer), TRUE), -1);
+									} else {
+										gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Disconnected</b></small>"), -1);
+									}
+									break;
+								case MMGUI_MAIN_TRAFFICLIST_ID_DATALIMIT:
+									if (device->connected) {
+										if (mmguiapp->coreoptions != NULL) {
+											if (!mmguiapp->coreoptions->trafficexecuted) {
+												if (mmguiapp->coreoptions->trafficenabled) {
+													limitleft = mmguiapp->coreoptions->trafficfull - (device->rxbytes + device->txbytes);
+													if (mmguiapp->coreoptions->trafficfull > (device->rxbytes + device->txbytes)) {
+														gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_bytes(limitleft, buffer, sizeof(buffer), TRUE), -1);
+													} else {
+														gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Limit</b></small>"), -1);
+													}
+												} else {
+													gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Disabled</b></small>"), -1);
+												}
+											} else {
+												gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Limit</b></small>"), -1);
+											}
+										} else {
+											gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Disabled</b></small>"), -1);
+										}
+									} else {
+										gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Disconnected</b></small>"), -1);
+									}
+									break;
+								case MMGUI_MAIN_TRAFFICLIST_ID_TIMELIMIT:
+									if (device->connected) {
+										if (mmguiapp->coreoptions != NULL) {
+											if (!mmguiapp->coreoptions->timeexecuted) {
+												if (mmguiapp->coreoptions->timeenabled) {
+													limitleft = mmguiapp->coreoptions->timefull - device->sessiontime;
+													if (mmguiapp->coreoptions->timefull > device->sessiontime) {
+														gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_time(limitleft, buffer, sizeof(buffer), TRUE), -1);
+													} else {
+														gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Limit</b></small>"), -1);
+													}
+												} else {
+													gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Disabled</b></small>"), -1);
+												}
+											} else {
+												gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Limit</b></small>"), -1);
+											}
+										} else {
+											gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Disabled</b></small>"), -1);
+										}
+									} else {
+										gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, _("<small><b>Disconnected</b></small>"), -1);
+									}
+									break;
+								case MMGUI_MAIN_TRAFFICLIST_ID_RXDATA_MONTH:
+									gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_bytes(trafficdb->monthrxbytes, buffer, sizeof(buffer), TRUE), -1);
+									break;
+								case MMGUI_MAIN_TRAFFICLIST_ID_TXDATA_MONTH:
+									gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_bytes(trafficdb->monthtxbytes, buffer, sizeof(buffer), TRUE), -1);
+									break;
+								case MMGUI_MAIN_TRAFFICLIST_ID_TIME_MONTH:
+									gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_time(trafficdb->monthduration, buffer, sizeof(buffer), TRUE), -1);
+									break;
+								case MMGUI_MAIN_TRAFFICLIST_ID_RXDATA_YEAR:
+									gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_bytes(trafficdb->yearrxbytes, buffer, sizeof(buffer), TRUE), -1);
+									break;
+								case MMGUI_MAIN_TRAFFICLIST_ID_TXDATA_YEAR:
+									gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_bytes(trafficdb->yeartxbytes, buffer, sizeof(buffer), TRUE), -1);
+									break;
+								case MMGUI_MAIN_TRAFFICLIST_ID_TIME_YEAR:
+									gtk_tree_store_set(GTK_TREE_STORE(model), &elementiter, MMGUI_MAIN_TRAFFICLIST_VALUE, mmgui_str_format_time(trafficdb->yearduration, buffer, sizeof(buffer), TRUE), -1);
+									break;
+								default:
+									break;
+							}
+						}
+						elementvalid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &elementiter);
+					} while (elementvalid);
+				}
+			}
+			sectionvalid = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &sectioniter);
+		}
 	}
-	//Update traffic graph
+	/*Update traffic graph*/
 	window = gtk_widget_get_window(mmguiapp->window->trafficdrawingarea);
 	visible = gtk_widget_get_visible(mmguiapp->window->trafficdrawingarea);
-	//Update only if needed
+	/*Update only if needed*/
 	if ((gtk_notebook_get_current_page(GTK_NOTEBOOK(mmguiapp->window->notebook)) == MMGUI_MAIN_PAGE_TRAFFIC) && (window != NULL) && (visible)) {
 		//TODO: Determine rectangle
 		gdk_window_invalidate_rect(window, NULL, FALSE);
@@ -1089,8 +1111,9 @@ void mmgui_main_traffic_list_init(mmgui_application_t mmguiapp)
 {
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
-	GtkListStore *store;
+	GtkTreeStore *store;
 	GtkTreeIter iter;
+	GtkTreeIter subiter;
 	
 	if (mmguiapp == NULL) return;
 	
@@ -1103,30 +1126,60 @@ void mmgui_main_traffic_list_init(mmgui_application_t mmguiapp)
 	column = gtk_tree_view_column_new_with_attributes(_("Value"), renderer, "markup", MMGUI_MAIN_TRAFFICLIST_VALUE, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(mmguiapp->window->trafficparamslist), column);
 	
-	store = gtk_list_store_new(MMGUI_MAIN_TRAFFICLIST_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT);
-		
-	gtk_list_store_append(store, &iter);
-	gtk_list_store_set(store, &iter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Received data</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_RXDATA, -1);
+	store = gtk_tree_store_new(MMGUI_MAIN_TRAFFICLIST_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT);
 	
-	gtk_list_store_append(store, &iter);
-	gtk_list_store_set(store, &iter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Transmitted data</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_TXDATA, -1);
+	gtk_tree_store_append(store, &iter, NULL);
+	gtk_tree_store_set(store, &iter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Session</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_CAPTION, -1);
 	
-	gtk_list_store_append(store, &iter);
-	gtk_list_store_set(store, &iter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Receive speed</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_RXSPEED, -1);
+	gtk_tree_store_append(store, &subiter, &iter);
+	gtk_tree_store_set(store, &subiter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Received data</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_RXDATA, -1);
 	
-	gtk_list_store_append(store, &iter);
-	gtk_list_store_set(store, &iter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Transmit speed</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_TXSPEED, -1);
+	gtk_tree_store_append(store, &subiter, &iter);
+	gtk_tree_store_set(store, &subiter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Transmitted data</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_TXDATA, -1);
 	
-	gtk_list_store_append(store, &iter);
-	gtk_list_store_set(store, &iter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Session time</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_TIME, -1);
+	gtk_tree_store_append(store, &subiter, &iter);
+	gtk_tree_store_set(store, &subiter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Receive speed</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_RXSPEED, -1);
 	
-	gtk_list_store_append(store, &iter);
-	gtk_list_store_set(store, &iter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Traffic left</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_DATALIMIT, -1);
+	gtk_tree_store_append(store, &subiter, &iter);
+	gtk_tree_store_set(store, &subiter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Transmit speed</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_TXSPEED, -1);
 	
-	gtk_list_store_append(store, &iter);
-	gtk_list_store_set(store, &iter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Time left</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_TIMELIMIT, -1);
+	gtk_tree_store_append(store, &subiter, &iter);
+	gtk_tree_store_set(store, &subiter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Session time</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_TIME, -1);
+	
+	gtk_tree_store_append(store, &subiter, &iter);
+	gtk_tree_store_set(store, &subiter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Traffic left</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_DATALIMIT, -1);
+	
+	gtk_tree_store_append(store, &subiter, &iter);
+	gtk_tree_store_set(store, &subiter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Time left</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_TIMELIMIT, -1);
+	
+	gtk_tree_store_append(store, &iter, NULL);
+	gtk_tree_store_set(store, &iter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Month</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_CAPTION, -1);
+	
+	gtk_tree_store_append(store, &subiter, &iter);
+	gtk_tree_store_set(store, &subiter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Received data</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_RXDATA_MONTH, -1);
+	
+	gtk_tree_store_append(store, &subiter, &iter);
+	gtk_tree_store_set(store, &subiter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Transmitted data</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_TXDATA_MONTH, -1);
+	
+	gtk_tree_store_append(store, &subiter, &iter);
+	gtk_tree_store_set(store, &subiter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Total time</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_TIME_MONTH, -1);
+	
+	gtk_tree_store_append(store, &iter, NULL);
+	gtk_tree_store_set(store, &iter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Year</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_CAPTION, -1);
+	
+	gtk_tree_store_append(store, &subiter, &iter);
+	gtk_tree_store_set(store, &subiter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Received data</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_RXDATA_YEAR, -1);
+	
+	gtk_tree_store_append(store, &subiter, &iter);
+	gtk_tree_store_set(store, &subiter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Transmitted data</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_TXDATA_YEAR, -1);
+	
+	gtk_tree_store_append(store, &subiter, &iter);
+	gtk_tree_store_set(store, &subiter, MMGUI_MAIN_TRAFFICLIST_PARAMETER, _("<small><b>Total time</b></small>"), MMGUI_MAIN_TRAFFICLIST_ID, MMGUI_MAIN_TRAFFICLIST_ID_TIME_YEAR, -1);
 	
 	gtk_tree_view_set_model(GTK_TREE_VIEW(mmguiapp->window->trafficparamslist), GTK_TREE_MODEL(store));
+	
+	gtk_tree_view_expand_all(GTK_TREE_VIEW(mmguiapp->window->trafficparamslist));
+	
 	g_object_unref(store);
 }
 
@@ -1150,16 +1203,16 @@ void mmgui_main_traffic_restore_settings_for_modem(mmgui_application_t mmguiapp)
 		
 	switch (mmguiapp->coreoptions->trafficunits) {
 		case 0:
-			mmguiapp->coreoptions->trafficfull = mmguiapp->coreoptions->trafficamount*1024*1024;
+			mmguiapp->coreoptions->trafficfull = (guint64)mmguiapp->coreoptions->trafficamount*1024*1024;
 			break;
 		case 1:
-			mmguiapp->coreoptions->trafficfull = mmguiapp->coreoptions->trafficamount*1024*1024*1024;
+			mmguiapp->coreoptions->trafficfull = (guint64)mmguiapp->coreoptions->trafficamount*1024*1024*1024;
 			break;
 		case 2:
-			mmguiapp->coreoptions->trafficfull = mmguiapp->coreoptions->trafficamount*1024*1024*1024*1024;
+			mmguiapp->coreoptions->trafficfull = (guint64)mmguiapp->coreoptions->trafficamount*1024*1024*1024*1024;
 			break;
 		default:
-			mmguiapp->coreoptions->trafficfull = mmguiapp->coreoptions->trafficamount*1024*1024;
+			mmguiapp->coreoptions->trafficfull = (guint64)mmguiapp->coreoptions->trafficamount*1024*1024;
 			break;
 	}
 	
