@@ -364,6 +364,7 @@ void mmgui_main_ussd_request_send(mmgui_application_t mmguiapp)
 	GtkTextBuffer *buffer;
 	GtkTextIter startiter, enditer;
 	GtkTextMark *position;
+	gchar *statusmsg;
 	
 	if (mmguiapp == NULL) return;
 	
@@ -383,27 +384,31 @@ void mmgui_main_ussd_request_send(mmgui_application_t mmguiapp)
 				if (validationid == MMGUI_USSD_VALIDATION_REQUEST) {
 					/*save last request*/
 					mmgui_modem_settings_set_string(mmguiapp->modemsettings, "ussd_sent_request", request);
-					//clear text view
+					/*clear text view*/
 					gtk_text_buffer_get_bounds(buffer, &startiter, &enditer);
 					gtk_text_buffer_delete(buffer, &startiter, &enditer);
-					//add request text
+					/*add request text*/
 					gtk_text_buffer_get_end_iter(buffer, &enditer);
 					gtk_text_buffer_insert_with_tags(buffer, &enditer, request, -1, mmguiapp->window->ussdrequesttag, NULL);
 					gtk_text_buffer_get_end_iter(buffer, &enditer);
 					gtk_text_buffer_insert_with_tags(buffer, &enditer, "\n", -1, mmguiapp->window->ussdrequesttag, NULL);
-				} else if (validationid == MMGUI_USSD_VALIDATION_RESPONSE) {
-					//add response text
+					statusmsg = g_strdup_printf(_("Sending USSD request %s..."), request);
+				} else {
+					/*add response text*/
 					gtk_text_buffer_get_end_iter(buffer, &enditer);
 					gtk_text_buffer_insert_with_tags(buffer, &enditer, request, -1, mmguiapp->window->ussdrequesttag, NULL);
 					gtk_text_buffer_get_end_iter(buffer, &enditer);
 					gtk_text_buffer_insert_with_tags(buffer, &enditer, "\n", -1, mmguiapp->window->ussdrequesttag, NULL);
+					statusmsg = g_strdup_printf(_("Sending USSD response %s..."), request);
 				}
-				//scroll to the end of buffer
+				/*scroll to the end of buffer*/
 				position = gtk_text_buffer_create_mark(buffer, "position", &enditer, FALSE);
 				gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW(mmguiapp->window->ussdtext), position);
 				gtk_text_buffer_delete_mark(buffer, position);
 				//show progress dialog
-				mmgui_main_ui_progress_dialog_open(mmguiapp);
+				//mmgui_main_ui_progress_dialog_open(mmguiapp);
+				mmgui_ui_infobar_show(mmguiapp, statusmsg, MMGUI_MAIN_INFOBAR_TYPE_PROGRESS);
+				g_free(statusmsg);
 			} else {
 				mmgui_main_ui_error_dialog_open(mmguiapp, _("<b>Error sending USSD</b>"), _("Wrong USSD request or device not ready"));
 			}
@@ -428,24 +433,32 @@ void mmgui_main_ussd_request_send_end(mmgui_application_t mmguiapp, mmguicore_t
 	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(mmguiapp->window->ussdtext));
 	
 	if (answer != NULL) {
-		//Add answer text
+		/*Add answer text*/
 		gtk_text_buffer_get_end_iter(buffer, &enditer);
 		gtk_text_buffer_insert_with_tags(buffer, &enditer, answer, -1, mmguiapp->window->ussdanswertag, NULL);
 		if (sessionstate == MMGUI_USSD_STATE_USER_RESPONSE) {
-			//Add session hint
+			/*Add session hint*/
 			gtk_text_buffer_get_end_iter(buffer, &enditer);
 			gtk_text_buffer_insert_with_tags(buffer, &enditer, _("\nUSSD session is active. Waiting for your input...\n"), -1, mmguiapp->window->ussdhinttag, NULL);
 		} else {
-			//Add new line symbol
+			/*Add new line symbol*/
 			gtk_text_buffer_get_end_iter(buffer, &enditer);
 			gtk_text_buffer_insert_with_tags(buffer, &enditer, "\n", -1, mmguiapp->window->ussdanswertag, NULL);
 		}
-		//scroll to the end of buffer
+		/*Scroll to the end of buffer*/
 		position = gtk_text_buffer_create_mark(buffer, "position", &enditer, FALSE);
 		gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW(mmguiapp->window->ussdtext), position);
 		gtk_text_buffer_delete_mark(buffer, position);
 	} else {
-		mmgui_main_ui_error_dialog_open(mmguiapp, _("<b>Error sending USSD</b>"), mmguicore_get_last_error(mmguiapp->core));
+		/*Add error hint*/
+		gtk_text_buffer_get_end_iter(buffer, &enditer);
+		gtk_text_buffer_insert_with_tags(buffer, &enditer, _("\nNo answer received..."), -1, mmguiapp->window->ussdhinttag, NULL);
+		/*Scroll to the end of buffer*/
+		position = gtk_text_buffer_create_mark(buffer, "position", &enditer, FALSE);
+		gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW(mmguiapp->window->ussdtext), position);
+		gtk_text_buffer_delete_mark(buffer, position);
+		/*Show menubar with error message*/
+		mmgui_ui_infobar_show_result(mmguiapp, MMGUI_MAIN_INFOBAR_RESULT_FAIL, mmguicore_get_last_error(mmguiapp->core));
 	}
 }
 
